@@ -10,6 +10,8 @@ use crate::isa::pulley_shared::abi::PulleyMachineDeps;
 use crate::{CodegenError, CodegenResult, settings};
 use crate::{machinst::*, trace};
 use alloc::string::{String, ToString};
+use alloc::vec;
+use alloc::vec::Vec;
 use regalloc2::RegClass;
 use smallvec::SmallVec;
 
@@ -148,7 +150,7 @@ fn pulley_get_operands(inst: &mut Inst, collector: &mut impl OperandVisitor) {
             collector.reg_def(dst);
         }
 
-        Inst::Call { info } | Inst::PatchableCall { info } => {
+        Inst::Call { info } => {
             let CallInfo {
                 uses,
                 defs,
@@ -435,8 +437,7 @@ where
             }
             | Inst::Call { .. }
             | Inst::IndirectCall { .. }
-            | Inst::IndirectCallHost { .. }
-            | Inst::PatchableCall { .. } => true,
+            | Inst::IndirectCallHost { .. } => true,
             _ => false,
         }
     }
@@ -499,10 +500,9 @@ where
 
     fn call_type(&self) -> CallType {
         match &self.inst {
-            Inst::Call { .. }
-            | Inst::IndirectCall { .. }
-            | Inst::IndirectCallHost { .. }
-            | Inst::PatchableCall { .. } => CallType::Regular,
+            Inst::Call { .. } | Inst::IndirectCall { .. } | Inst::IndirectCallHost { .. } => {
+                CallType::Regular
+            }
 
             Inst::ReturnCall { .. } | Inst::ReturnIndirectCall { .. } => CallType::TailCall,
 
@@ -535,14 +535,14 @@ where
         todo!()
     }
 
-    fn gen_nop_unit() -> SmallVec<[u8; 8]> {
-        let mut bytes = smallvec::smallvec![];
+    fn gen_nop_units() -> Vec<Vec<u8>> {
+        let mut bytes = vec![];
         let nop = pulley_interpreter::op::Nop {};
         nop.encode(&mut bytes);
         // NOP needs to be a 1-byte opcode so it can be used to
         // overwrite a callsite of any length.
         assert_eq!(bytes.len(), 1);
-        bytes
+        vec![bytes]
     }
 
     fn rc_for_type(ty: Type) -> CodegenResult<(&'static [RegClass], &'static [Type])> {
@@ -611,7 +611,7 @@ const TRAP_OPCODE: &'static [u8] = &[
 
 #[test]
 fn test_trap_encoding() {
-    let mut dst = std::vec::Vec::new();
+    let mut dst = alloc::vec::Vec::new();
     pulley_interpreter::encode::trap(&mut dst);
     assert_eq!(dst, TRAP_OPCODE);
 }
@@ -724,10 +724,6 @@ impl Inst {
                 format!("indirect_call {callee}, {info:?}{try_call}")
             }
 
-            Inst::PatchableCall { info } => {
-                format!("patchable_call {info:?}")
-            }
-
             Inst::ReturnCall { info } => {
                 format!("return_call {info:?}")
             }
