@@ -408,8 +408,8 @@ pub(crate) fn emit(
             Inst::addq_mi(rsp, guard_plus_count).emit(sink, info, state);
         }
 
-        Inst::CallKnown { info: call_info } | Inst::PatchableCallKnown { info: call_info } => {
-            let is_patchable = matches!(inst, Inst::PatchableCallKnown { .. });
+        Inst::CallKnown { info: call_info } => {
+            let start = sink.cur_offset();
             let stack_map = state.take_stack_map();
 
             asm::inst::callq_d::new(0).emit(sink, info, state);
@@ -431,10 +431,6 @@ pub(crate) fn emit(
                     Some(state.frame_layout().sp_to_fp()),
                     try_call.exception_handlers(&state.frame_layout()),
                 );
-            } else if is_patchable {
-                // Patchable call-site of length 5 bytes. The
-                // MachBuffer knows how to NOP out the callsite.
-                sink.add_patchable_call_site(5);
             } else {
                 sink.add_call_site();
             }
@@ -443,19 +439,22 @@ pub(crate) fn emit(
             // callee, to ensure that StackAMode values are always computed from
             // a consistent SP.
             if call_info.callee_pop_size > 0 {
-                assert!(!is_patchable);
                 let rsp = Writable::from_reg(regs::rsp());
                 let callee_pop_size = i32::try_from(call_info.callee_pop_size)
                     .expect("`callee_pop_size` is too large to fit in a 32-bit immediate");
                 Inst::subq_mi(rsp, callee_pop_size).emit(sink, info, state);
             }
 
-            // Load any stack-carried return values.
-            call_info.emit_retval_loads::<X64ABIMachineSpec, _, _>(
-                state.frame_layout().stackslots_size,
-                |inst| inst.emit(sink, info, state),
-                |_space_needed| None,
-            );
+            if call_info.patchable {
+                sink.add_patchable_call_site(sink.cur_offset() - start);
+            } else {
+                // Load any stack-carried return values.
+                call_info.emit_retval_loads::<X64ABIMachineSpec, _, _>(
+                    state.frame_layout().stackslots_size,
+                    |inst| inst.emit(sink, info, state),
+                    |_space_needed| None,
+                );
+            }
 
             // If this is a try-call, jump to the continuation
             // (normal-return) block.
@@ -736,7 +735,7 @@ pub(crate) fn emit(
             // Emit jump table (table of 32-bit offsets).
             sink.bind_label(start_of_jumptable, state.ctrl_plane_mut());
             let jt_off = sink.cur_offset();
-            for &target in targets.iter().chain(std::iter::once(default_target)) {
+            for &target in targets.iter().chain(core::iter::once(default_target)) {
                 let word_off = sink.cur_offset();
                 // off_into_table is an addend here embedded in the label to be later patched at
                 // the end of codegen. The offset is initially relative to this jump table entry;
