@@ -291,8 +291,6 @@ pub enum ModuleError {
 
     /// Memory allocation failure from a backend
     Allocation {
-        /// Tell where the allocation came from
-        message: &'static str,
         /// Io error the allocation failed with
         err: std::io::Error,
     },
@@ -321,7 +319,7 @@ impl std::error::Error for ModuleError {
             | Self::DuplicateDefinition { .. }
             | Self::InvalidImportDefinition { .. } => None,
             Self::Compilation(source) => Some(source),
-            Self::Allocation { err: source, .. } => Some(source),
+            Self::Allocation { err: source } => Some(source),
             Self::Backend(source) => Some(&**source),
             Self::Flag(source) => Some(source),
         }
@@ -355,8 +353,8 @@ impl std::fmt::Display for ModuleError {
             Self::Compilation(err) => {
                 write!(f, "Compilation error: {err}")
             }
-            Self::Allocation { message, err } => {
-                write!(f, "Allocation error: {message}: {err}")
+            Self::Allocation { err } => {
+                write!(f, "Allocation error: {err}")
             }
             Self::Backend(err) => write!(f, "Backend error: {err}"),
             Self::Flag(err) => write!(f, "Flag error: {err}"),
@@ -915,6 +913,7 @@ pub trait Module {
             name: ir::ExternalName::user(user_name_ref),
             signature,
             colocated,
+            patchable: false,
         })
     }
 
