@@ -15,11 +15,11 @@ use object::write::{
     Object, Relocation, SectionId, StandardSection, Symbol, SymbolId, SymbolSection,
 };
 use object::{
-    RelocationEncoding, RelocationFlags, RelocationKind, SectionKind, SymbolFlags, SymbolKind,
-    SymbolScope,
+    RelocationEncoding, RelocationFlags, RelocationKind, SectionFlags, SectionKind, SymbolFlags,
+    SymbolKind, SymbolScope, elf,
 };
-use std::collections::hash_map::Entry;
 use std::collections::HashMap;
+use std::collections::hash_map::Entry;
 use std::mem;
 use target_lexicon::PointerWidth;
 
@@ -57,16 +57,15 @@ impl ObjectBuilder {
             target_lexicon::BinaryFormat::Wasm => {
                 return Err(ModuleError::Backend(anyhow!(
                     "binary format wasm is unsupported",
-                )))
+                )));
             }
             target_lexicon::BinaryFormat::Unknown => {
-                return Err(ModuleError::Backend(anyhow!("binary format is unknown")))
+                return Err(ModuleError::Backend(anyhow!("binary format is unknown")));
             }
             other => {
                 return Err(ModuleError::Backend(anyhow!(
-                    "binary format {} not recognized",
-                    other
-                )))
+                    "binary format {other} not recognized"
+                )));
             }
         };
         let architecture = match isa.triple().architecture {
@@ -77,8 +76,7 @@ impl ObjectBuilder {
             target_lexicon::Architecture::Riscv64(_) => {
                 if binary_format != object::BinaryFormat::Elf {
                     return Err(ModuleError::Backend(anyhow!(
-                        "binary format {:?} is not supported for riscv64",
-                        binary_format,
+                        "binary format {binary_format:?} is not supported for riscv64",
                     )));
                 }
 
@@ -105,9 +103,8 @@ impl ObjectBuilder {
             target_lexicon::Architecture::S390x => object::Architecture::S390x,
             architecture => {
                 return Err(ModuleError::Backend(anyhow!(
-                    "target architecture {:?} is unsupported",
-                    architecture,
-                )))
+                    "target architecture {architecture:?} is unsupported",
+                )));
             }
         };
         let endian = match isa.triple().endianness().unwrap() {
@@ -187,8 +184,7 @@ fn validate_symbol(name: &str) -> ModuleResult<()> {
     // crate to panic. Let's return a clean error instead.
     if name.contains("\0") {
         return Err(ModuleError::Backend(anyhow::anyhow!(
-            "Symbol {:?} has a null byte, which is disallowed",
-            name
+            "Symbol {name:?} has a null byte, which is disallowed"
         )));
     }
     Ok(())
@@ -394,6 +390,7 @@ impl Module for ObjectModule {
             data_relocs: _,
             ref custom_segment_section,
             align,
+            used,
         } = data;
 
         let pointer_reloc = match self.isa.triple().pointer_width().unwrap() {
@@ -422,7 +419,7 @@ impl Module for ObjectModule {
             } else {
                 StandardSection::ReadOnlyDataWithRel
             };
-            if self.per_data_object_section {
+            if self.per_data_object_section || used {
                 // FIXME pass empty symbol name once add_subsection produces `.text` as section name
                 // instead of `.text.` when passed an empty symbol name. (object#748) Until then
                 // pass `subsection` to produce `.text.subsection` as section name to reduce
@@ -451,6 +448,21 @@ impl Module for ObjectModule {
             )
         };
 
+        if used {
+            match self.object.format() {
+                object::BinaryFormat::Elf => match self.object.section_flags_mut(section) {
+                    SectionFlags::Elf { sh_flags } => *sh_flags |= u64::from(elf::SHF_GNU_RETAIN),
+                    _ => unreachable!(),
+                },
+                object::BinaryFormat::Coff => {}
+                object::BinaryFormat::MachO => match self.object.symbol_flags_mut(symbol) {
+                    SymbolFlags::MachO { n_desc } => *n_desc |= object::macho::N_NO_DEAD_STRIP,
+                    _ => unreachable!(),
+                },
+                _ => unreachable!(),
+            }
+        }
+
         let align = std::cmp::max(align.unwrap_or(1), self.isa.symbol_alignment());
         let offset = match *init {
             Init::Uninitialized => {
@@ -482,7 +494,7 @@ impl ObjectModule {
         bytes: &[u8],
         relocs: Vec<ObjectRelocRecord>,
     ) -> Result<(), ModuleError> {
-        info!("defining function {} with bytes", func_id);
+        info!("defining function {func_id} with bytes");
         let decl = self.declarations.get_function_decl(func_id);
         let decl_name = decl.linkage_name(func_id);
         if !decl.linkage.is_definable() {
@@ -520,6 +532,34 @@ impl ObjectModule {
 
     /// Finalize all relocations and output an object.
     pub fn finish(mut self) -> ObjectProduct {
+        if cfg!(debug_assertions) {
+            for (func_id, decl) in self.declarations.get_functions() {
+                if !decl.linkage.requires_definition() {
+                    continue;
+                }
+
+                assert!(
+                    self.functions[func_id].unwrap().1,
+                    "function \"{}\" with linkage {:?} must be defined but is not",
+                    decl.linkage_name(func_id),
+                    decl.linkage,
+                );
+            }
+
+            for (data_id, decl) in self.declarations.get_data_objects() {
+                if !decl.linkage.requires_definition() {
+                    continue;
+                }
+
+                assert!(
+                    self.data_objects[data_id].unwrap().1,
+                    "data object \"{}\" with linkage {:?} must be defined but is not",
+                    decl.linkage_name(data_id),
+                    decl.linkage,
+                );
+            }
+        }
+
         let symbol_relocs = mem::take(&mut self.relocs);
         for symbol in symbol_relocs {
             for &ObjectRelocRecord {
