@@ -61,26 +61,6 @@ impl LinkOptions {
         self
     }
 }
-pub enum Baz {}
-#[wasmtime::component::__internal::trait_variant_make(::core::marker::Send)]
-pub trait HostBaz: Sized {
-    async fn foo(&mut self, self_: wasmtime::component::Resource<Baz>) -> ();
-    async fn drop(
-        &mut self,
-        rep: wasmtime::component::Resource<Baz>,
-    ) -> wasmtime::Result<()>;
-}
-impl<_T: HostBaz + ?Sized + Send> HostBaz for &mut _T {
-    async fn foo(&mut self, self_: wasmtime::component::Resource<Baz>) -> () {
-        HostBaz::foo(*self, self_).await
-    }
-    async fn drop(
-        &mut self,
-        rep: wasmtime::component::Resource<Baz>,
-    ) -> wasmtime::Result<()> {
-        HostBaz::drop(*self, rep).await
-    }
-}
 impl core::convert::From<LinkOptions> for foo::foo::the_interface::LinkOptions {
     fn from(src: LinkOptions) -> Self {
         (&src).into()
@@ -98,6 +78,36 @@ impl core::convert::From<&LinkOptions> for foo::foo::the_interface::LinkOptions
         dest
     }
 }
+pub enum Baz {}
+pub trait HostBazWithStore: wasmtime::component::HasData + Send {}
+impl<_T: ?Sized> HostBazWithStore for _T
+where
+    _T: wasmtime::component::HasData + Send,
+{}
+pub trait HostBaz: Send {
+    fn foo(
+        &mut self,
+        self_: wasmtime::component::Resource<Baz>,
+    ) -> impl ::core::future::Future<Output = ()> + Send;
+    fn drop(
+        &mut self,
+        rep: wasmtime::component::Resource<Baz>,
+    ) -> impl ::core::future::Future<Output = wasmtime::Result<()>> + Send;
+}
+impl<_T: HostBaz + ?Sized + Send> HostBaz for &mut _T {
+    fn foo(
+        &mut self,
+        self_: wasmtime::component::Resource<Baz>,
+    ) -> impl ::core::future::Future<Output = ()> + Send {
+        async move { HostBaz::foo(*self, self_).await }
+    }
+    async fn drop(
+        &mut self,
+        rep: wasmtime::component::Resource<Baz>,
+    ) -> wasmtime::Result<()> {
+        HostBaz::drop(*self, rep).await
+    }
+}
 /// Auto-generated bindings for a pre-instantiated version of a
 /// component which implements the world `the-world`.
 ///
@@ -106,11 +116,11 @@ impl core::convert::From<&LinkOptions> for foo::foo::the_interface::LinkOptions
 /// has been created through a [`Linker`](wasmtime::component::Linker).
 ///
 /// For more information see [`TheWorld`] as well.
-pub struct TheWorldPre<T> {
+pub struct TheWorldPre<T: 'static> {
     instance_pre: wasmtime::component::InstancePre<T>,
     indices: TheWorldIndices,
 }
-impl<T> Clone for TheWorldPre<T> {
+impl<T: 'static> Clone for TheWorldPre<T> {
     fn clone(&self) -> Self {
         Self {
             instance_pre: self.instance_pre.clone(),
@@ -118,7 +128,7 @@ impl<T> Clone for TheWorldPre<T> {
         }
     }
 }
-impl<_T> TheWorldPre<_T> {
+impl<_T: 'static> TheWorldPre<_T> {
     /// Creates a new copy of `TheWorldPre` bindings which can then
     /// be used to instantiate into a particular store.
     ///
@@ -127,7 +137,7 @@ impl<_T> TheWorldPre<_T> {
     pub fn new(
         instance_pre: wasmtime::component::InstancePre<_T>,
     ) -> wasmtime::Result<Self> {
-        let indices = TheWorldIndices::new(instance_pre.component())?;
+        let indices = TheWorldIndices::new(&instance_pre)?;
         Ok(Self { instance_pre, indices })
     }
     pub fn engine(&self) -> &wasmtime::Engine {
@@ -143,13 +153,21 @@ impl<_T> TheWorldPre<_T> {
     /// instance to perform instantiation. Afterwards the preloaded
     /// indices in `self` are used to lookup all exports on the
     /// resulting instance.
+    pub fn instantiate(
+        &self,
+        mut store: impl wasmtime::AsContextMut<Data = _T>,
+    ) -> wasmtime::Result<TheWorld> {
+        let mut store = store.as_context_mut();
+        let instance = self.instance_pre.instantiate(&mut store)?;
+        self.indices.load(&mut store, &instance)
+    }
+}
+impl<_T: Send + 'static> TheWorldPre<_T> {
+    /// Same as [`Self::instantiate`], except with `async`.
     pub async fn instantiate_async(
         &self,
         mut store: impl wasmtime::AsContextMut<Data = _T>,
-    ) -> wasmtime::Result<TheWorld>
-    where
-        _T: Send,
-    {
+    ) -> wasmtime::Result<TheWorld> {
         let mut store = store.as_context_mut();
         let instance = self.instance_pre.instantiate_async(&mut store).await?;
         self.indices.load(&mut store, &instance)
@@ -171,23 +189,18 @@ pub struct TheWorldIndices {}
 /// depending on your requirements and what you have on hand:
 ///
 /// * The most convenient way is to use
-///   [`TheWorld::instantiate_async`] which only needs a
+///   [`TheWorld::instantiate`] which only needs a
 ///   [`Store`], [`Component`], and [`Linker`].
 ///
 /// * Alternatively you can create a [`TheWorldPre`] ahead of
 ///   time with a [`Component`] to front-load string lookups
 ///   of exports once instead of per-instantiation. This
-///   method then uses [`TheWorldPre::instantiate_async`] to
+///   method then uses [`TheWorldPre::instantiate`] to
 ///   create a [`TheWorld`].
 ///
 /// * If you've instantiated the instance yourself already
 ///   then you can use [`TheWorld::new`].
 ///
-/// * You can also access the guts of instantiation through
-///   [`TheWorldIndices::new_instance`] followed
-///   by [`TheWorldIndices::load`] to crate an instance of this
-///   type.
-///
 /// These methods are all equivalent to one another and move
 /// around the tradeoff of what work is performed when.
 ///
@@ -195,26 +208,17 @@ pub struct TheWorldIndices {}
 /// [`Component`]: wasmtime::component::Component
 /// [`Linker`]: wasmtime::component::Linker
 pub struct TheWorld {}
-#[wasmtime::component::__internal::trait_variant_make(::core::marker::Send)]
-pub trait TheWorldImports: Send + HostBaz {
-    async fn foo(&mut self) -> ();
-}
-pub trait TheWorldImportsGetHost<
-    T,
-    D,
->: Fn(T) -> <Self as TheWorldImportsGetHost<T, D>>::Host + Send + Sync + Copy + 'static {
-    type Host: TheWorldImports;
-}
-impl<F, T, D, O> TheWorldImportsGetHost<T, D> for F
+pub trait TheWorldImportsWithStore: wasmtime::component::HasData + HostBazWithStore + Send {}
+impl<_T: ?Sized> TheWorldImportsWithStore for _T
 where
-    F: Fn(T) -> O + Send + Sync + Copy + 'static,
-    O: TheWorldImports,
-{
-    type Host = O;
+    _T: wasmtime::component::HasData + HostBazWithStore + Send,
+{}
+pub trait TheWorldImports: HostBaz + Send {
+    fn foo(&mut self) -> impl ::core::future::Future<Output = ()> + Send;
 }
 impl<_T: TheWorldImports + ?Sized + Send> TheWorldImports for &mut _T {
-    async fn foo(&mut self) -> () {
-        TheWorldImports::foo(*self).await
+    fn foo(&mut self) -> impl ::core::future::Future<Output = ()> + Send {
+        async move { TheWorldImports::foo(*self).await }
     }
 }
 const _: () = {
@@ -226,24 +230,11 @@ const _: () = {
         ///
         /// This method may fail if the component does not have the
         /// required exports.
-        pub fn new(
-            component: &wasmtime::component::Component,
+        pub fn new<_T>(
+            _instance_pre: &wasmtime::component::InstancePre<_T>,
         ) -> wasmtime::Result<Self> {
-            let _component = component;
-            Ok(TheWorldIndices {})
-        }
-        /// Creates a new instance of [`TheWorldIndices`] from an
-        /// instantiated component.
-        ///
-        /// This method of creating a [`TheWorld`] will perform string
-        /// lookups for all exports when this method is called. This
-        /// will only succeed if the provided instance matches the
-        /// requirements of [`TheWorld`].
-        pub fn new_instance(
-            mut store: impl wasmtime::AsContextMut,
-            instance: &wasmtime::component::Instance,
-        ) -> wasmtime::Result<Self> {
-            let _instance = instance;
+            let _component = _instance_pre.component();
+            let _instance_type = _instance_pre.instance_type();
             Ok(TheWorldIndices {})
         }
         /// Uses the indices stored in `self` to load an instance
@@ -256,15 +247,35 @@ const _: () = {
             mut store: impl wasmtime::AsContextMut,
             instance: &wasmtime::component::Instance,
         ) -> wasmtime::Result<TheWorld> {
+            let _ = &mut store;
             let _instance = instance;
             Ok(TheWorld {})
         }
     }
     impl TheWorld {
+        /// Convenience wrapper around [`TheWorldPre::new`] and
+        /// [`TheWorldPre::instantiate`].
+        pub fn instantiate<_T>(
+            store: impl wasmtime::AsContextMut<Data = _T>,
+            component: &wasmtime::component::Component,
+            linker: &wasmtime::component::Linker<_T>,
+        ) -> wasmtime::Result<TheWorld> {
+            let pre = linker.instantiate_pre(component)?;
+            TheWorldPre::new(pre)?.instantiate(store)
+        }
+        /// Convenience wrapper around [`TheWorldIndices::new`] and
+        /// [`TheWorldIndices::load`].
+        pub fn new(
+            mut store: impl wasmtime::AsContextMut,
+            instance: &wasmtime::component::Instance,
+        ) -> wasmtime::Result<TheWorld> {
+            let indices = TheWorldIndices::new(&instance.instance_pre(&store))?;
+            indices.load(&mut store, instance)
+        }
         /// Convenience wrapper around [`TheWorldPre::new`] and
         /// [`TheWorldPre::instantiate_async`].
         pub async fn instantiate_async<_T>(
-            mut store: impl wasmtime::AsContextMut<Data = _T>,
+            store: impl wasmtime::AsContextMut<Data = _T>,
             component: &wasmtime::component::Component,
             linker: &wasmtime::component::Linker<_T>,
         ) -> wasmtime::Result<TheWorld>
@@ -274,25 +285,15 @@ const _: () = {
             let pre = linker.instantiate_pre(component)?;
             TheWorldPre::new(pre)?.instantiate_async(store).await
         }
-        /// Convenience wrapper around [`TheWorldIndices::new_instance`] and
-        /// [`TheWorldIndices::load`].
-        pub fn new(
-            mut store: impl wasmtime::AsContextMut,
-            instance: &wasmtime::component::Instance,
-        ) -> wasmtime::Result<TheWorld> {
-            let indices = TheWorldIndices::new_instance(&mut store, instance)?;
-            indices.load(store, instance)
-        }
-        pub fn add_to_linker_imports_get_host<
-            T,
-            G: for<'a> TheWorldImportsGetHost<&'a mut T, T, Host: TheWorldImports>,
-        >(
+        pub fn add_to_linker_imports<T, D>(
             linker: &mut wasmtime::component::Linker<T>,
             options: &LinkOptions,
-            host_getter: G,
+            host_getter: fn(&mut T) -> D::Data<'_>,
         ) -> wasmtime::Result<()>
         where
-            T: Send,
+            D: TheWorldImportsWithStore,
+            for<'a> D::Data<'a>: TheWorldImports,
+            T: 'static + Send,
         {
             let mut linker = linker.root();
             if options.experimental_world {
@@ -344,23 +345,23 @@ const _: () = {
             }
             Ok(())
         }
-        pub fn add_to_linker<T, U>(
+        pub fn add_to_linker<T, D>(
             linker: &mut wasmtime::component::Linker<T>,
             options: &LinkOptions,
-            get: impl Fn(&mut T) -> &mut U + Send + Sync + Copy + 'static,
+            host_getter: fn(&mut T) -> D::Data<'_>,
         ) -> wasmtime::Result<()>
         where
-            T: Send,
-            U: foo::foo::the_interface::Host + TheWorldImports + Send,
+            D: foo::foo::the_interface::HostWithStore + TheWorldImportsWithStore + Send,
+            for<'a> D::Data<'a>: foo::foo::the_interface::Host + TheWorldImports + Send,
+            T: 'static + Send,
         {
             if options.experimental_world {
-                Self::add_to_linker_imports_get_host(linker, options, get)?;
+                Self::add_to_linker_imports::<T, D>(linker, options, host_getter)?;
                 if options.experimental_world_interface_import {
-                    foo::foo::the_interface::add_to_linker(
-                        linker,
-                        &options.into(),
-                        get,
-                    )?;
+                    foo::foo::the_interface::add_to_linker::<
+                        T,
+                        D,
+                    >(linker, &options.into(), host_getter)?;
                 }
             }
             Ok(())
@@ -413,20 +414,27 @@ pub mod foo {
                 }
             }
             pub enum Bar {}
-            #[wasmtime::component::__internal::trait_variant_make(::core::marker::Send)]
-            pub trait HostBar: Sized {
-                async fn foo(&mut self, self_: wasmtime::component::Resource<Bar>) -> ();
-                async fn drop(
-                    &mut self,
-                    rep: wasmtime::component::Resource<Bar>,
-                ) -> wasmtime::Result<()>;
-            }
-            impl<_T: HostBar + ?Sized + Send> HostBar for &mut _T {
-                async fn foo(
+            pub trait HostBarWithStore: wasmtime::component::HasData + Send {}
+            impl<_T: ?Sized> HostBarWithStore for _T
+            where
+                _T: wasmtime::component::HasData + Send,
+            {}
+            pub trait HostBar: Send {
+                fn foo(
                     &mut self,
                     self_: wasmtime::component::Resource<Bar>,
-                ) -> () {
-                    HostBar::foo(*self, self_).await
+                ) -> impl ::core::future::Future<Output = ()> + Send;
+                fn drop(
+                    &mut self,
+                    rep: wasmtime::component::Resource<Bar>,
+                ) -> impl ::core::future::Future<Output = wasmtime::Result<()>> + Send;
+            }
+            impl<_T: HostBar + ?Sized + Send> HostBar for &mut _T {
+                fn foo(
+                    &mut self,
+                    self_: wasmtime::component::Resource<Bar>,
+                ) -> impl ::core::future::Future<Output = ()> + Send {
+                    async move { HostBar::foo(*self, self_).await }
                 }
                 async fn drop(
                     &mut self,
@@ -435,33 +443,28 @@ pub mod foo {
                     HostBar::drop(*self, rep).await
                 }
             }
-            #[wasmtime::component::__internal::trait_variant_make(::core::marker::Send)]
-            pub trait Host: Send + HostBar + Sized {
-                async fn foo(&mut self) -> ();
-            }
-            pub trait GetHost<
-                T,
-                D,
-            >: Fn(T) -> <Self as GetHost<T, D>>::Host + Send + Sync + Copy + 'static {
-                type Host: Host + Send;
-            }
-            impl<F, T, D, O> GetHost<T, D> for F
+            pub trait HostWithStore: wasmtime::component::HasData + HostBarWithStore + Send {}
+            impl<_T: ?Sized> HostWithStore for _T
             where
-                F: Fn(T) -> O + Send + Sync + Copy + 'static,
-                O: Host + Send,
-            {
-                type Host = O;
+                _T: wasmtime::component::HasData + HostBarWithStore + Send,
+            {}
+            pub trait Host: HostBar + Send {
+                fn foo(&mut self) -> impl ::core::future::Future<Output = ()> + Send;
             }
-            pub fn add_to_linker_get_host<
-                T,
-                G: for<'a> GetHost<&'a mut T, T, Host: Host + Send>,
-            >(
+            impl<_T: Host + ?Sized + Send> Host for &mut _T {
+                fn foo(&mut self) -> impl ::core::future::Future<Output = ()> + Send {
+                    async move { Host::foo(*self).await }
+                }
+            }
+            pub fn add_to_linker<T, D>(
                 linker: &mut wasmtime::component::Linker<T>,
                 options: &LinkOptions,
-                host_getter: G,
+                host_getter: fn(&mut T) -> D::Data<'_>,
             ) -> wasmtime::Result<()>
             where
-                T: Send,
+                D: HostWithStore,
+                for<'a> D::Data<'a>: Host,
+                T: 'static + Send,
             {
                 if options.experimental_interface {
                     let mut inst = linker.instance("foo:foo/the-interface")?;
@@ -510,22 +513,6 @@ pub mod foo {
                 }
                 Ok(())
             }
-            pub fn add_to_linker<T, U>(
-                linker: &mut wasmtime::component::Linker<T>,
-                options: &LinkOptions,
-                get: impl Fn(&mut T) -> &mut U + Send + Sync + Copy + 'static,
-            ) -> wasmtime::Result<()>
-            where
-                U: Host + Send,
-                T: Send,
-            {
-                add_to_linker_get_host(linker, options, get)
-            }
-            impl<_T: Host + ?Sized + Send> Host for &mut _T {
-                async fn foo(&mut self) -> () {
-                    Host::foo(*self).await
-                }
-            }
         }
     }
 }
