@@ -28,9 +28,9 @@
 //! fused adapters, what arguments make their way to core wasm modules, etc.
 
 use crate::component::*;
+use crate::error::Result;
 use crate::prelude::*;
 use crate::{EntityIndex, EntityRef, ModuleInternedTypeIndex, PrimaryMap, WasmValType};
-use anyhow::Result;
 use cranelift_entity::packed_option::PackedOption;
 use indexmap::IndexMap;
 use info::LinearMemoryOptions;
@@ -338,9 +338,6 @@ pub enum Trampoline {
         instance: RuntimeComponentInstanceIndex,
         ty: TypeResourceTableIndex,
     },
-    BackpressureSet {
-        instance: RuntimeComponentInstanceIndex,
-    },
     BackpressureInc {
         instance: RuntimeComponentInstanceIndex,
     },
@@ -479,6 +476,7 @@ pub enum Trampoline {
     StreamTransfer,
     ErrorContextTransfer,
     CheckBlocking,
+    Trap,
     ContextGet {
         instance: RuntimeComponentInstanceIndex,
         slot: u32,
@@ -959,9 +957,6 @@ impl LinearizeDfg<'_> {
                 instance: *instance,
                 ty: *ty,
             },
-            Trampoline::BackpressureSet { instance } => info::Trampoline::BackpressureSet {
-                instance: *instance,
-            },
             Trampoline::BackpressureInc { instance } => info::Trampoline::BackpressureInc {
                 instance: *instance,
             },
@@ -1162,6 +1157,7 @@ impl LinearizeDfg<'_> {
             Trampoline::StreamTransfer => info::Trampoline::StreamTransfer,
             Trampoline::ErrorContextTransfer => info::Trampoline::ErrorContextTransfer,
             Trampoline::CheckBlocking => info::Trampoline::CheckBlocking,
+            Trampoline::Trap => info::Trampoline::Trap,
             Trampoline::ContextGet { instance, slot } => info::Trampoline::ContextGet {
                 instance: *instance,
                 slot: *slot,
