@@ -1,16 +1,16 @@
 use crate::Abi;
 use crate::component::dfg::AbstractInstantiations;
 use crate::component::*;
+use crate::error::Context;
+use crate::error::anyhow;
+use crate::error::ensure;
+use crate::error::{Result, bail};
 use crate::prelude::*;
 use crate::{
     EngineOrModuleTypeIndex, EntityIndex, FuncKey, ModuleEnvironment, ModuleInternedTypeIndex,
     ModuleTranslation, ModuleTypesBuilder, PrimaryMap, ScopeVec, TagIndex, Tunables, TypeConvert,
     WasmHeapType, WasmResult, WasmValType,
 };
-use anyhow::Context;
-use anyhow::anyhow;
-use anyhow::ensure;
-use anyhow::{Result, bail};
 use core::str::FromStr;
 use cranelift_entity::SecondaryMap;
 use cranelift_entity::packed_option::PackedOption;
@@ -199,9 +199,6 @@ enum LocalInitializer<'data> {
     ResourceRep(AliasableResourceId, ModuleInternedTypeIndex),
     ResourceDrop(AliasableResourceId, ModuleInternedTypeIndex),
 
-    BackpressureSet {
-        func: ModuleInternedTypeIndex,
-    },
     BackpressureInc {
         func: ModuleInternedTypeIndex,
     },
@@ -871,9 +868,7 @@ impl<'a, 'data> Translator<'a, 'data> {
                             bail!("unsupported intrinsic")
                         }
                         wasmparser::CanonicalFunction::BackpressureSet => {
-                            let core_type = self.core_func_signature(core_func_index)?;
-                            core_func_index += 1;
-                            LocalInitializer::BackpressureSet { func: core_type }
+                            bail!("unsupported intrinsic")
                         }
                         wasmparser::CanonicalFunction::BackpressureInc => {
                             let core_type = self.core_func_signature(core_func_index)?;
