@@ -181,27 +181,16 @@ pub fn smoke_test_engine<T>(
 ) where
     T: DiffEngine,
 {
-    use rand::prelude::*;
-
-    let mut rng = SmallRng::seed_from_u64(0);
-    let mut buf = vec![0; 2048];
-    let n = 100;
-    for _ in 0..n {
-        rng.fill_bytes(&mut buf);
-        let mut u = Unstructured::new(&buf);
-        let mut config = match u.arbitrary::<Config>() {
-            Ok(config) => config,
-            Err(_) => continue,
-        };
+    crate::test::test_n_times(5, |mut config: Config, u| {
         // This will ensure that wasmtime, which uses this configuration
         // settings, can guaranteed instantiate a module.
         config.set_differential_config();
 
-        let mut engine = match mk_engine(&mut u, &mut config) {
+        let mut engine = match mk_engine(u, &mut config) {
             Ok(engine) => engine,
             Err(e) => {
                 println!("skip {e:?}");
-                continue;
+                return Ok(());
             }
         };
 
@@ -240,8 +229,6 @@ pub fn smoke_test_engine<T>(
             }
         }
 
-        return;
-    }
-
-    panic!("after {n} runs nothing ever ran, something is probably wrong");
+        Ok(())
+    })
 }
