@@ -19,7 +19,7 @@ use wasmtime::Store;
 use wasmtime::component::{Component, Linker, ResourceTable};
 use wasmtime_wasi::p3::bindings::Command;
 use wasmtime_wasi::{TrappableError, WasiCtx, WasiCtxBuilder, WasiCtxView, WasiView};
-use wasmtime_wasi_http::p3::bindings::Proxy;
+use wasmtime_wasi_http::p3::bindings::Service;
 use wasmtime_wasi_http::p3::bindings::http::types::ErrorCode;
 use wasmtime_wasi_http::p3::{
     self, Request, RequestOptions, WasiHttpCtx, WasiHttpCtxView, WasiHttpView,
@@ -144,6 +144,7 @@ async fn run_cli(path: &str, server: &Server) -> anyhow::Result<()> {
         .await
         .context("failed to call `wasi:cli/run#run`")?
         .context("guest trapped")?
+        .0
         .map_err(|()| anyhow!("`wasi:cli/run#run` failed"))
 }
 
@@ -167,7 +168,7 @@ async fn run_http<E: Into<ErrorCode> + 'static>(
     wasmtime_wasi::p3::add_to_linker(&mut linker).context("failed to link `wasi:cli@0.3.x`")?;
     wasmtime_wasi_http::p3::add_to_linker(&mut linker)
         .context("failed to link `wasi:http@0.3.x`")?;
-    let proxy = Proxy::instantiate_async(&mut store, &component, &linker).await?;
+    let service = Service::instantiate_async(&mut store, &component, &linker).await?;
     let (req, io) = Request::from_http(req);
     let (tx, rx) = tokio::sync::oneshot::channel();
     let ((handle_result, ()), res) = try_join!(
@@ -176,7 +177,7 @@ async fn run_http<E: Into<ErrorCode> + 'static>(
                 .run_concurrent(async |store| {
                     try_join!(
                         async {
-                            let (res, task) = match proxy.handle(store, req).await? {
+                            let (res, task) = match service.handle(store, req).await? {
                                 Ok(pair) => pair,
                                 Err(err) => return Ok(Err(Some(err))),
                             };
@@ -333,7 +334,7 @@ async fn p3_http_middleware() -> Result<()> {
 async fn p3_http_middleware_host_to_host() {
     let error = format!("{:?}", test_http_middleware(true).await.unwrap_err());
 
-    let expected = "cannot read from and write to intra-component stream with non-unit payload";
+    let expected = "cannot read from and write to intra-component future with non-numeric payload";
 
     assert!(
         error.contains(expected),
@@ -360,6 +361,8 @@ async fn compose(a: &[u8], b: &[u8]) -> Result<Vec<u8>> {
     let b_file = dir.path().join("b.wasm");
     fs::write(&b_file, b).await?;
 
+    // The middleware imports `wasi:http/handler` which matches the echo's
+    // `wasi:http/handler` export, so wasm-compose can link them automatically.
     ComponentComposer::new(
         &a_file,
         &wasm_compose::config::Config {
@@ -412,7 +415,7 @@ async fn test_http_middleware_with_chain(host_to_host: bool) -> Result<()> {
                         "local:local/chain-http".to_owned(),
                         InstantiationArg {
                             instance: "local:local/chain-http".into(),
-                            export: Some("wasi:http/handler@0.3.0-rc-2025-09-16".into()),
+                            export: Some("wasi:http/handler@0.3.0-rc-2026-01-06".into()),
                         },
                     )]
                     .into_iter()
