@@ -2,11 +2,11 @@
 
 use super::{AnyRef, RootedGcRefImpl};
 use crate::prelude::*;
-use crate::runtime::vm::VMGcRef;
+use crate::runtime::vm::{self, VMGcRef, VMStore};
 use crate::{
-    store::{AutoAssertNoGc, StoreOpaque},
-    AsContextMut, GcHeapOutOfMemory, GcRefImpl, GcRootIndex, HeapType, ManuallyRooted, RefType,
+    AsContextMut, GcHeapOutOfMemory, GcRefImpl, GcRootIndex, HeapType, OwnedRooted, RefType,
     Result, Rooted, StoreContext, StoreContextMut, ValRaw, ValType, WasmTy,
+    store::{AutoAssertNoGc, StoreOpaque, StoreResourceLimiter},
 };
 use core::any::Any;
 use core::mem;
@@ -31,7 +31,7 @@ use core::mem::MaybeUninit;
 /// the host into dereferencing it and segfaulting or worse.
 ///
 /// Note that you can also use `Rooted<ExternRef>` and
-/// `ManuallyRooted<ExternRef>` as a type parameter with
+/// `OwnedRooted<ExternRef>` as a type parameter with
 /// [`Func::typed`][crate::Func::typed]- and
 /// [`Func::wrap`][crate::Func::wrap]-style APIs.
 ///
@@ -119,7 +119,6 @@ pub struct ExternRef {
 }
 
 unsafe impl GcRefImpl for ExternRef {
-    #[allow(private_interfaces)]
     fn transmute_ref(index: &GcRootIndex) -> &Self {
         // Safety: `ExternRef` is a newtype of a `GcRootIndex`.
         let me: &Self = unsafe { mem::transmute(index) };
@@ -137,28 +136,33 @@ unsafe impl GcRefImpl for ExternRef {
 }
 
 impl ExternRef {
-    /// Creates a new instance of `ExternRef` wrapping the given value.
+    /// Synchronously allocates a new `ExternRef` wrapping the given value.
     ///
     /// The resulting value is automatically unrooted when the given `context`'s
-    /// scope is exited. See [`Rooted<T>`][crate::Rooted]'s documentation for
-    /// more details.
+    /// scope is exited. If you need to hold the reference past the `context`'s
+    /// scope, convert the result into an
+    /// [`OwnedRooted<T>`][crate::OwnedRooted]. See the documentation for
+    /// [`Rooted<T>`][crate::Rooted] and
+    /// [`OwnedRooted<T>`][crate::OwnedRooted] for more details.
     ///
-    /// This method will *not* automatically trigger a GC to free up space in
-    /// the GC heap; instead it will return an error. This gives you more
-    /// precise control over when collections happen and allows you to choose
-    /// between performing synchronous and asynchronous collections.
+    /// # Automatic Garbage Collection
+    ///
+    /// If the GC heap is at capacity, and there isn't room for allocating a new
+    /// `externref`, this method will automatically trigger a synchronous
+    /// collection in an attempt to free up space in the GC heap.
     ///
     /// # Errors
     ///
     /// If the allocation cannot be satisfied because the GC heap is currently
-    /// out of memory, but performing a garbage collection might free up space
-    /// such that retrying the allocation afterwards might succeed, then a
-    /// `GcHeapOutOfMemory<T>` error is returned.
+    /// out of memory, then a [`GcHeapOutOfMemory<T>`][crate::GcHeapOutOfMemory]
+    /// error is returned. The allocation might succeed on a second attempt if
+    /// you drop some rooted GC references and try again.
     ///
-    /// The `GcHeapOutOfMemory<T>` error contains the host value that the
-    /// `externref` would have wrapped. You can extract that value from this
-    /// error and reuse it when attempting to allocate an `externref` again
-    /// after GC or otherwise do with it whatever you see fit.
+    /// The [`GcHeapOutOfMemory<T>`][crate::GcHeapOutOfMemory] error contains
+    /// the host value that the `externref` would have wrapped. You can extract
+    /// that value from the error and reuse it when attempting to allocate an
+    /// `externref` again after dropping rooted GC references and then
+    /// performing a collection or otherwise do with it whatever you see fit.
     ///
     /// # Example
     ///
@@ -170,26 +174,27 @@ impl ExternRef {
     /// {
     ///     let mut scope = RootScope::new(&mut store);
     ///
-    ///     // Create an `externref` wrapping a `str`.
-    ///     let externref = match ExternRef::new(&mut scope, "hello!") {
+    ///     // Allocate an `externref` wrapping a `String`.
+    ///     let externref = match ExternRef::new(&mut scope, "hello!".to_string()) {
+    ///         // The allocation succeeded.
     ///         Ok(x) => x,
-    ///         // If the heap is out of memory, then do a GC and try again.
-    ///         Err(e) if e.is::<GcHeapOutOfMemory<&'static str>>() => {
-    ///             // Do a GC! Note: in an async context, you'd want to do
-    ///             // `scope.as_context_mut().gc_async().await`.
-    ///             scope.as_context_mut().gc();
-    ///
-    ///             // Extract the original host value from the error.
-    ///             let host_value = e
-    ///                 .downcast::<GcHeapOutOfMemory<&'static str>>()
-    ///                 .unwrap()
-    ///                 .into_inner();
-    ///
-    ///             // Try to allocate the `externref` again, now that the GC
-    ///             // has hopefully freed up some space.
-    ///             ExternRef::new(&mut scope, host_value)?
-    ///         }
-    ///         Err(e) => return Err(e),
+    ///         // The allocation failed.
+    ///         Err(e) => match e.downcast::<GcHeapOutOfMemory<String>>() {
+    ///             // The allocation failed because the GC heap does not have
+    ///             // capacity for this allocation.
+    ///             Ok(oom) => {
+    ///                 // Take back ownership of our `String`.
+    ///                 let s = oom.into_inner();
+    ///                 // Drop some rooted GC refs from our system to potentially
+    ///                 // free up space for Wasmtime to make this allocation.
+    /// #               let drop_some_gc_refs = || {};
+    ///                 drop_some_gc_refs();
+    ///                 // Finally, try to allocate again, reusing the original
+    ///                 // string.
+    ///                 ExternRef::new(&mut scope, s)?
+    ///             }
+    ///             Err(e) => return Err(e),
+    ///         },
     ///     };
     ///
     ///     // Use the `externref`, pass it to Wasm, etc...
@@ -199,21 +204,136 @@ impl ExternRef {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn new<T>(mut context: impl AsContextMut, value: T) -> Result<Rooted<ExternRef>>
+    ///
+    /// # Panics
+    ///
+    /// Panics if the `context` is configured for async; use
+    /// [`ExternRef::new_async`][crate::ExternRef::new_async] to perform
+    /// asynchronous allocation instead.
+    pub fn new<T>(mut store: impl AsContextMut, value: T) -> Result<Rooted<ExternRef>>
     where
         T: 'static + Any + Send + Sync,
     {
-        let ctx = context.as_context_mut().0;
+        let (mut limiter, store) = store.as_context_mut().0.resource_limiter_and_store_opaque();
+        assert!(!store.async_support());
+        vm::assert_ready(Self::_new_async(store, limiter.as_mut(), value))
+    }
 
+    /// Asynchronously allocates a new `ExternRef` wrapping the given value.
+    ///
+    /// The resulting value is automatically unrooted when the given `context`'s
+    /// scope is exited. If you need to hold the reference past the `context`'s
+    /// scope, convert the result into an
+    /// [`OwnedRooted<T>`][crate::OwnedRooted]. See the documentation for
+    /// [`Rooted<T>`][crate::Rooted] and
+    /// [`OwnedRooted<T>`][crate::OwnedRooted] for more details.
+    ///
+    /// # Automatic Garbage Collection
+    ///
+    /// If the GC heap is at capacity, and there isn't room for allocating a new
+    /// `externref`, this method will automatically trigger an asynchronous
+    /// collection in an attempt to free up space in the GC heap.
+    ///
+    /// # Errors
+    ///
+    /// If the allocation cannot be satisfied because the GC heap is currently
+    /// out of memory, then a [`GcHeapOutOfMemory<T>`][crate::GcHeapOutOfMemory]
+    /// error is returned. The allocation might succeed on a second attempt if
+    /// you drop some rooted GC references and try again.
+    ///
+    /// The [`GcHeapOutOfMemory<T>`][crate::GcHeapOutOfMemory] error contains
+    /// the host value that the `externref` would have wrapped. You can extract
+    /// that value from the error and reuse it when attempting to allocate an
+    /// `externref` again after dropping rooted GC references and then
+    /// performing a collection or otherwise do with it whatever you see fit.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use wasmtime::*;
+    ///
+    /// # async fn _foo() -> Result<()> {
+    /// let mut store = Store::<()>::default();
+    ///
+    /// {
+    ///     let mut scope = RootScope::new(&mut store);
+    ///
+    ///     // Create an `externref` wrapping a `String`.
+    ///     let externref = match ExternRef::new_async(&mut scope, "hello!".to_string()).await {
+    ///         // The allocation succeeded.
+    ///         Ok(x) => x,
+    ///         // The allocation failed.
+    ///         Err(e) => match e.downcast::<GcHeapOutOfMemory<String>>() {
+    ///             // The allocation failed because the GC heap does not have
+    ///             // capacity for this allocation.
+    ///             Ok(oom) => {
+    ///                 // Take back ownership of our `String`.
+    ///                 let s = oom.into_inner();
+    ///                 // Drop some rooted GC refs from our system to potentially
+    ///                 // free up space for Wasmtime to make this allocation.
+    /// #               let drop_some_gc_refs = || {};
+    ///                 drop_some_gc_refs();
+    ///                 // Finally, try to allocate again, reusing the original
+    ///                 // string.
+    ///                 ExternRef::new_async(&mut scope, s).await?
+    ///             }
+    ///             Err(e) => return Err(e),
+    ///         },
+    ///     };
+    ///
+    ///     // Use the `externref`, pass it to Wasm, etc...
+    /// }
+    ///
+    /// // The `externref` is automatically unrooted when we exit the scope.
+    /// # Ok(())
+    /// # }
+    /// ```
+    ///
+    /// # Panics
+    ///
+    /// Panics if the `context` is not configured for async; use
+    /// [`ExternRef::new`][crate::ExternRef::new] to perform synchronous
+    /// allocation instead.
+    #[cfg(feature = "async")]
+    pub async fn new_async<T>(mut store: impl AsContextMut, value: T) -> Result<Rooted<ExternRef>>
+    where
+        T: 'static + Any + Send + Sync,
+    {
+        let (mut limiter, store) = store.as_context_mut().0.resource_limiter_and_store_opaque();
+        Self::_new_async(store, limiter.as_mut(), value).await
+    }
+
+    pub(crate) async fn _new_async<T>(
+        store: &mut StoreOpaque,
+        limiter: Option<&mut StoreResourceLimiter<'_>>,
+        value: T,
+    ) -> Result<Rooted<ExternRef>>
+    where
+        T: 'static + Any + Send + Sync,
+    {
+        // Allocate the box once, regardless how many gc-and-retry attempts we
+        // make.
         let value: Box<dyn Any + Send + Sync> = Box::new(value);
-        let gc_ref = ctx
-            .gc_store_mut()?
-            .alloc_externref(value)
-            .context("unrecoverable error when allocating new `externref`")?
-            .map_err(|x| GcHeapOutOfMemory::<T>::new(*x.downcast().unwrap()))
-            .context("failed to allocate `externref`")?;
 
-        let mut ctx = AutoAssertNoGc::new(ctx);
+        let gc_ref = store
+            .retry_after_gc_async(limiter, value, |store, value| {
+                store
+                    .require_gc_store_mut()?
+                    .alloc_externref(value)
+                    .context("unrecoverable error when allocating new `externref`")?
+                    .map_err(|(x, n)| GcHeapOutOfMemory::new(x, n).into())
+            })
+            .await
+            // Translate the `GcHeapOutOfMemory`'s inner value from the boxed
+            // trait object into `T`.
+            .map_err(
+                |e| match e.downcast::<GcHeapOutOfMemory<Box<dyn Any + Send + Sync>>>() {
+                    Ok(oom) => oom.map_inner(|x| *x.downcast::<T>().unwrap()).into(),
+                    Err(e) => e,
+                },
+            )?;
+
+        let mut ctx = AutoAssertNoGc::new(store);
         Ok(Self::from_cloned_gc_ref(&mut ctx, gc_ref.into()))
     }
 
@@ -274,58 +394,6 @@ impl ExternRef {
         Ok(Self::from_cloned_gc_ref(store, gc_ref))
     }
 
-    /// Creates a new, manually-rooted instance of `ExternRef` wrapping the
-    /// given value.
-    ///
-    /// The resulting value must be manually unrooted, or else it will leak for
-    /// the entire duration of the store's lifetime. See
-    /// [`ManuallyRooted<T>`][crate::ManuallyRooted]'s documentation for more
-    /// details.
-    ///
-    /// # Errors
-    ///
-    /// This function returns the same errors in the same scenarios as
-    /// [`ExternRef::new`][crate::ExternRef::new].
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # use wasmtime::*;
-    /// # fn _foo() -> Result<()> {
-    /// let mut store = Store::<()>::default();
-    ///
-    /// // Create a manually-rooted `externref` wrapping a `str`.
-    /// let externref = ExternRef::new_manually_rooted(&mut store, "hello!")?;
-    ///
-    /// // Use `externref` a bunch, pass it to Wasm, etc...
-    ///
-    /// // Don't forget to explicitly unroot the `externref` when you're done
-    /// // using it!
-    /// externref.unroot(&mut store);
-    /// # Ok(())
-    /// # }
-    /// ```
-    pub fn new_manually_rooted<T>(
-        mut store: impl AsContextMut,
-        value: T,
-    ) -> Result<ManuallyRooted<ExternRef>>
-    where
-        T: 'static + Any + Send + Sync,
-    {
-        let ctx = store.as_context_mut().0;
-
-        let value: Box<dyn Any + Send + Sync> = Box::new(value);
-        let gc_ref = ctx
-            .gc_store_mut()?
-            .alloc_externref(value)
-            .context("unrecoverable error when allocating new `externref`")?
-            .map_err(|x| GcHeapOutOfMemory::<T>::new(*x.downcast().unwrap()))
-            .context("failed to allocate `externref`")?;
-
-        let mut ctx = AutoAssertNoGc::new(ctx);
-        Ok(ManuallyRooted::new(&mut ctx, gc_ref.into()))
-    }
-
     /// Create a new `Rooted<ExternRef>` from the given GC reference.
     ///
     /// Does not invoke the `GcRuntime`'s clone hook; callers should ensure it
@@ -339,11 +407,13 @@ impl ExternRef {
         store: &mut AutoAssertNoGc<'_>,
         gc_ref: VMGcRef,
     ) -> Rooted<Self> {
-        assert!(
-            gc_ref.is_extern_ref(&*store.unwrap_gc_store().gc_heap)
-                || gc_ref.is_any_ref(&*store.unwrap_gc_store().gc_heap),
-            "GC reference {gc_ref:#p} should be an externref or anyref"
-        );
+        if !gc_ref.is_i31() {
+            assert!(
+                gc_ref.is_extern_ref(&*store.unwrap_gc_store().gc_heap)
+                    || gc_ref.is_any_ref(&*store.unwrap_gc_store().gc_heap),
+                "GC reference {gc_ref:#p} should be an externref or anyref"
+            );
+        }
         Rooted::new(store, gc_ref)
     }
 
@@ -381,11 +451,14 @@ impl ExternRef {
         store: impl Into<StoreContext<'a, T>>,
     ) -> Result<Option<&'a (dyn Any + Send + Sync)>>
     where
-        T: 'a,
+        T: 'static,
     {
         let store = store.into().0;
         let gc_ref = self.inner.try_gc_ref(&store)?;
-        let gc_store = store.gc_store()?;
+        if gc_ref.is_i31() {
+            return Ok(None);
+        }
+        let gc_store = store.require_gc_store()?;
         if let Some(externref) = gc_ref.as_externref(&*gc_store.gc_heap) {
             Ok(Some(gc_store.externref_host_data(externref)))
         } else {
@@ -429,14 +502,17 @@ impl ExternRef {
         store: impl Into<StoreContextMut<'a, T>>,
     ) -> Result<Option<&'a mut (dyn Any + Send + Sync)>>
     where
-        T: 'a,
+        T: 'static,
     {
         let store = store.into().0;
         // NB: need to do an unchecked copy to release the borrow on the store
         // so that we can get the store's GC store. But importantly we cannot
         // trigger a GC while we are working with `gc_ref` here.
         let gc_ref = self.inner.try_gc_ref(store)?.unchecked_copy();
-        let gc_store = store.gc_store_mut()?;
+        if gc_ref.is_i31() {
+            return Ok(None);
+        }
+        let gc_store = store.require_gc_store_mut()?;
         if let Some(externref) = gc_ref.as_externref(&*gc_store.gc_heap) {
             Ok(Some(gc_store.externref_host_data_mut(externref)))
         } else {
@@ -452,27 +528,30 @@ impl ExternRef {
     /// This function assumes that `raw` is an externref value which is
     /// currently rooted within the [`Store`].
     ///
-    /// # Unsafety
+    /// # Correctness
     ///
-    /// This function is particularly `unsafe` because `raw` not only must be a
-    /// valid externref value produced prior by `to_raw` but it must also be
-    /// correctly rooted within the store. When arguments are provided to a
-    /// callback with [`Func::new_unchecked`], for example, or returned via
-    /// [`Func::call_unchecked`], if a GC is performed within the store then
-    /// floating externref values are not rooted and will be GC'd, meaning that
-    /// this function will no longer be safe to call with the values cleaned up.
-    /// This function must be invoked *before* possible GC operations can happen
-    /// (such as calling wasm).
+    /// This function is tricky to get right because `raw` not only must be a
+    /// valid `externref` value produced prior by [`ExternRef::to_raw`] but it
+    /// must also be correctly rooted within the store. When arguments are
+    /// provided to a callback with [`Func::new_unchecked`], for example, or
+    /// returned via [`Func::call_unchecked`], if a GC is performed within the
+    /// store then floating `externref` values are not rooted and will be GC'd,
+    /// meaning that this function will no longer be correct to call with the
+    /// values cleaned up. This function must be invoked *before* possible GC
+    /// operations can happen (such as calling Wasm).
     ///
-    /// When in doubt try to not use this. Instead use the safe Rust APIs of
-    /// [`TypedFunc`] and friends.
+    ///
+    /// When in doubt try to not use this. Instead use the Rust APIs of
+    /// [`TypedFunc`] and friends. Note though that this function is not
+    /// `unsafe` as any value can be passed in. Incorrect values can result in
+    /// runtime panics, however, so care must still be taken with this method.
     ///
     /// [`Func::call_unchecked`]: crate::Func::call_unchecked
     /// [`Func::new_unchecked`]: crate::Func::new_unchecked
     /// [`Store`]: crate::Store
     /// [`TypedFunc`]: crate::TypedFunc
     /// [`ValRaw`]: crate::ValRaw
-    pub unsafe fn from_raw(mut store: impl AsContextMut, raw: u32) -> Option<Rooted<ExternRef>> {
+    pub fn from_raw(mut store: impl AsContextMut, raw: u32) -> Option<Rooted<ExternRef>> {
         let mut store = AutoAssertNoGc::new(store.as_context_mut().0);
         Self::_from_raw(&mut store, raw)
     }
@@ -480,7 +559,7 @@ impl ExternRef {
     // (Not actually memory unsafe since we have indexed GC heaps.)
     pub(crate) fn _from_raw(store: &mut AutoAssertNoGc, raw: u32) -> Option<Rooted<ExternRef>> {
         let gc_ref = VMGcRef::from_raw_u32(raw)?;
-        let gc_ref = store.unwrap_gc_store_mut().clone_gc_ref(&gc_ref);
+        let gc_ref = store.clone_gc_ref(&gc_ref);
         Some(Self::from_cloned_gc_ref(store, gc_ref))
     }
 
@@ -489,14 +568,14 @@ impl ExternRef {
     ///
     /// Returns an error if this `externref` has been unrooted.
     ///
-    /// # Unsafety
+    /// # Correctness
     ///
-    /// Produces a raw value which is only safe to pass into a store if a GC
+    /// Produces a raw value which is only valid to pass into a store if a GC
     /// doesn't happen between when the value is produce and when it's passed
     /// into the store.
     ///
     /// [`ValRaw`]: crate::ValRaw
-    pub unsafe fn to_raw(&self, mut store: impl AsContextMut) -> Result<u32> {
+    pub fn to_raw(&self, mut store: impl AsContextMut) -> Result<u32> {
         let mut store = AutoAssertNoGc::new(store.as_context_mut().0);
         self._to_raw(&mut store)
     }
@@ -567,7 +646,7 @@ unsafe impl WasmTy for Option<Rooted<ExternRef>> {
     }
 }
 
-unsafe impl WasmTy for ManuallyRooted<ExternRef> {
+unsafe impl WasmTy for OwnedRooted<ExternRef> {
     #[inline]
     fn valtype() -> ValType {
         ValType::Ref(RefType::new(false, HeapType::Extern))
@@ -597,7 +676,7 @@ unsafe impl WasmTy for ManuallyRooted<ExternRef> {
     }
 }
 
-unsafe impl WasmTy for Option<ManuallyRooted<ExternRef>> {
+unsafe impl WasmTy for Option<OwnedRooted<ExternRef>> {
     #[inline]
     fn valtype() -> ValType {
         ValType::EXTERNREF
@@ -620,11 +699,11 @@ unsafe impl WasmTy for Option<ManuallyRooted<ExternRef>> {
     }
 
     fn store(self, store: &mut AutoAssertNoGc<'_>, ptr: &mut MaybeUninit<ValRaw>) -> Result<()> {
-        <ManuallyRooted<ExternRef>>::wasm_ty_option_store(self, store, ptr, ValRaw::externref)
+        <OwnedRooted<ExternRef>>::wasm_ty_option_store(self, store, ptr, ValRaw::externref)
     }
 
     unsafe fn load(store: &mut AutoAssertNoGc<'_>, ptr: &ValRaw) -> Self {
-        <ManuallyRooted<ExternRef>>::wasm_ty_option_load(
+        <OwnedRooted<ExternRef>>::wasm_ty_option_load(
             store,
             ptr.get_externref(),
             ExternRef::from_cloned_gc_ref,
