@@ -5,7 +5,7 @@ use crate::component::Instance;
 use crate::component::concurrent::WaitResult;
 use crate::prelude::*;
 #[cfg(feature = "component-model-async")]
-use crate::runtime::component::concurrent::ResourcePair;
+use crate::runtime::component::concurrent::{ResourcePair, RuntimeInstance};
 use crate::runtime::vm::component::{ComponentInstance, VMComponentContext};
 use crate::runtime::vm::{HostResultHasUnwindSentinel, VMStore, VmSafe};
 use core::cell::Cell;
@@ -665,32 +665,24 @@ fn resource_exit_call(store: &mut dyn VMStore, instance: Instance) -> Result<()>
     instance.resource_exit_call(store)
 }
 
-fn trap(_store: &mut dyn VMStore, _instance: Instance, code: u8) -> Result<()> {
-    Err(wasmtime_environ::Trap::from_u8(code).unwrap().into())
-}
-
-#[cfg(feature = "component-model-async")]
-fn backpressure_set(
-    store: &mut dyn VMStore,
-    _instance: Instance,
-    caller_instance: u32,
-    enabled: u32,
-) -> Result<()> {
-    store.concurrent_state_mut().backpressure_modify(
-        RuntimeComponentInstanceIndex::from_u32(caller_instance),
-        |_| Some(if enabled != 0 { 1 } else { 0 }),
-    )
+fn trap(_store: &mut dyn VMStore, _instance: Instance, code: u32) -> Result<()> {
+    Err(wasmtime_environ::Trap::from_u8(u8::try_from(code).unwrap())
+        .unwrap()
+        .into())
 }
 
 #[cfg(feature = "component-model-async")]
 fn backpressure_modify(
     store: &mut dyn VMStore,
-    _instance: Instance,
+    instance: Instance,
     caller_instance: u32,
     increment: u8,
 ) -> Result<()> {
-    store.concurrent_state_mut().backpressure_modify(
-        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+    store.backpressure_modify(
+        RuntimeInstance {
+            instance: instance.id().instance(),
+            index: RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        },
         |old| {
             if increment != 0 {
                 old.checked_add(1)
