@@ -35,14 +35,14 @@ class Heap:
     self.memory[ret : ret + original_size] = self.memory[original_ptr : original_ptr + original_size]
     return ret
 
-def mk_opts(memory = bytearray(), encoding = 'utf8', realloc = None, post_return = None, sync_task_return = False, sync = True):
+def mk_opts(memory = bytearray(), encoding = 'utf8', realloc = None, post_return = None, sync_task_return = False, async_ = False):
   opts = CanonicalOptions()
   opts.memory = memory
   opts.string_encoding = encoding
   opts.realloc = realloc
   opts.post_return = post_return
   opts.sync_task_return = sync_task_return
-  opts.sync = sync
+  opts.async_ = async_
   opts.callback = None
   return opts
 
@@ -59,7 +59,7 @@ def run_lift(opts, inst, ft, callee, on_start, on_resolve):
 
 def mk_task(caller, on_resolve, thread_func):
   inst = ComponentInstance(caller.inst.store)
-  task = Task(None, inst, None, caller, on_resolve)
+  task = Task(None, inst, FuncType([],[],async_=True), caller, on_resolve)
   thread = Thread(task, thread_func)
   thread.resume()
   return task
@@ -497,7 +497,7 @@ def test_handles():
     assert((canon_resource_rep(rt, thread, h4))[0] == 45)
 
     dtor_value = None
-    canon_resource_drop(rt, True, thread, h1)
+    [] = canon_resource_drop(rt, thread, h1)
     assert(dtor_value == 42)
     assert(len(inst.table.array) == 6)
     assert(inst.table.array[h1] is None)
@@ -510,7 +510,7 @@ def test_handles():
     assert(len(inst.table.free) == 0)
 
     dtor_value = None
-    canon_resource_drop(rt, True, thread, h3)
+    [] = canon_resource_drop(rt, thread, h3)
     assert(dtor_value is None)
     assert(len(inst.table.array) == 6)
     assert(inst.table.array[h3] is None)
@@ -552,19 +552,19 @@ def test_handles():
 def test_async_to_async():
   producer_heap = Heap(10)
   producer_opts = mk_opts(producer_heap.memory)
-  producer_opts.sync = False
+  producer_opts.async_ = True
 
   store = Store()
   producer_inst = ComponentInstance(store)
 
-  eager_ft = FuncType([], [U8Type()])
+  eager_ft = FuncType([], [U8Type()], async_=True)
   def core_eager_producer(thread, args):
     assert(len(args) == 0)
     [] = canon_task_return(thread, [U8Type()], producer_opts, [43])
     return []
   eager_callee = partial(canon_lift, producer_opts, producer_inst, eager_ft, core_eager_producer)
 
-  toggle_ft = FuncType([], [])
+  toggle_ft = FuncType([], [], async_=True)
   fut1_1 = RacyBool(False)
   fut1_2 = RacyBool(False)
   def core_toggle(thread, args):
@@ -578,7 +578,7 @@ def test_async_to_async():
   toggle_callee = partial(canon_lift, producer_opts, producer_inst, toggle_ft, core_toggle)
 
   fut2, fut3, fut4 = RacyBool(False), RacyBool(False), RacyBool(False)
-  blocking_ft = FuncType([U8Type()], [U8Type()])
+  blocking_ft = FuncType([U8Type()], [U8Type()], async_=True)
   def core_blocking_producer(thread, args):
     [x] = args
     assert(x == 83)
@@ -591,7 +591,7 @@ def test_async_to_async():
 
   consumer_heap = Heap(20)
   consumer_opts = mk_opts(consumer_heap.memory)
-  consumer_opts.sync = False
+  consumer_opts.async_ = True
 
   def consumer(thread, args):
     [b] = args
@@ -640,39 +640,10 @@ def test_async_to_async():
     fut3.set()
     thread.suspend_until(fut4.is_set)
 
-    dtor_fut = RacyBool(False)
-    dtor_value = None
-    def dtor(thread, args):
-      nonlocal dtor_value
-      assert(len(args) == 1)
-      thread.suspend_until(dtor_fut.is_set)
-      dtor_value = args[0]
-      return []
-    rt = ResourceType(producer_inst, dtor)
-
-    [resi] = canon_resource_new(rt, thread, 50)
-    assert(resi == 4)
-    assert(dtor_value is None)
-    [ret] = canon_resource_drop(rt, False, thread, resi)
-    state,dtorsubi = unpack_result(ret)
-    assert(dtorsubi == 4)
-    assert(state == Subtask.State.STARTED)
-    assert(dtor_value is None)
-    dtor_fut.set()
-
-    [] = canon_waitable_join(thread, dtorsubi, seti)
-    [event] = canon_waitable_set_wait(True, consumer_heap.memory, thread, seti, waitretp)
-    assert(event == EventCode.SUBTASK)
-    assert(consumer_heap.memory[waitretp] == dtorsubi)
-    assert(consumer_heap.memory[waitretp+4] == Subtask.State.RETURNED)
-    assert(dtor_value == 50)
-    [] = canon_subtask_drop(thread, dtorsubi)
-    [] = canon_waitable_set_drop(thread, seti)
-
     [] = canon_task_return(thread, [U8Type()], consumer_opts, [42])
     return []
 
-  ft = FuncType([BoolType()],[U8Type()])
+  ft = FuncType([BoolType()],[U8Type()], async_=True)
 
   def on_start():
     return [ True ]
@@ -692,8 +663,8 @@ def test_async_callback():
   store = Store()
   producer_inst = ComponentInstance(store)
   producer_opts = mk_opts()
-  producer_opts.sync = False
-  producer_ft = FuncType([], [])
+  producer_opts.async_ = True
+  producer_ft = FuncType([], [], async_ = True)
 
   def core_producer_pre(fut, thread, args):
     assert(len(args) == 0)
@@ -707,7 +678,7 @@ def test_async_callback():
   core_producer2 = partial(core_producer_pre, fut2)
   producer2 = partial(canon_lift, producer_opts, producer_inst, producer_ft, core_producer2)
 
-  consumer_ft = FuncType([],[U32Type()])
+  consumer_ft = FuncType([],[U32Type()], async_ = True)
   seti = 0
   def consumer(thread, args):
     assert(len(args) == 0)
@@ -771,7 +742,7 @@ def test_async_callback():
     got = results
 
   opts = mk_opts()
-  opts.sync = False
+  opts.async_ = True
   opts.callback = callback
 
   run_lift(opts, consumer_inst, consumer_ft, consumer, on_start, on_resolve)
@@ -781,7 +752,7 @@ def test_async_callback():
 def test_callback_interleaving():
   store = Store()
   producer_inst = ComponentInstance(store)
-  producer_ft = FuncType([U32Type(), FutureType(None),FutureType(None),FutureType(None)],[U32Type()])
+  producer_ft = FuncType([U32Type(), FutureType(None),FutureType(None),FutureType(None)],[U32Type()], async_ = True)
   fut3s = [None,None]
   def core_producer(thread, args):
     [i,fut1,fut2,fut3] = args
@@ -795,7 +766,7 @@ def test_callback_interleaving():
 
     [seti] = canon_waitable_set_new(thread)
 
-    async_opts = mk_opts(sync = False)
+    async_opts = mk_opts(async_ = True)
     [ret] = canon_future_read(FutureType(None), async_opts, thread, fut2, 0xdeadbeef)
     assert(ret == definitions.BLOCKED)
 
@@ -817,11 +788,11 @@ def test_callback_interleaving():
 
     return [CallbackCode.EXIT]
   producer_opts = mk_opts()
-  producer_opts.sync = False
+  producer_opts.async_ = True
   producer_opts.callback = core_producer_callback
   producer_callee = partial(canon_lift, producer_opts, producer_inst, producer_ft, core_producer)
 
-  sync_callee_ft = FuncType([], [U32Type()])
+  sync_callee_ft = FuncType([], [U32Type()], async_ = True)
   def core_sync_callee(thread, args):
     assert(len(args) == 0)
     return [100]
@@ -829,9 +800,9 @@ def test_callback_interleaving():
   sync_callee = partial(canon_lift, sync_callee_opts, producer_inst, sync_callee_ft, core_sync_callee)
 
   consumer_inst = ComponentInstance(store)
-  consumer_ft = FuncType([], [])
+  consumer_ft = FuncType([], [], async_ = True)
   consumer_mem = bytearray(24)
-  consumer_opts = mk_opts(consumer_mem, sync = False)
+  consumer_opts = mk_opts(consumer_mem, async_ = True)
   def core_consumer(thread, args):
     assert(len(args) == 0)
 
@@ -852,7 +823,7 @@ def test_callback_interleaving():
     [ret] = canon_lower(consumer_opts, producer_ft, producer_callee, thread, [0, rfut11, rfut12, rfut13, 0xdeadbeef])
     state,todie = unpack_result(ret)
     assert(state == Subtask.State.STARTING)
-    [ret] = canon_subtask_cancel(True, thread, todie)
+    [ret] = canon_subtask_cancel(False, thread, todie)
     assert(ret == Subtask.State.CANCELLED_BEFORE_STARTED)
     producer_inst.backpressure = False
 
@@ -865,7 +836,7 @@ def test_callback_interleaving():
     state,todie = unpack_result(ret)
     assert(state == Subtask.State.STARTING)
 
-    [ret] = canon_subtask_cancel(True, thread, todie)
+    [ret] = canon_subtask_cancel(False, thread, todie)
     assert(ret == Subtask.State.CANCELLED_BEFORE_STARTED)
 
     subi2ret = 16
@@ -949,12 +920,69 @@ def test_callback_interleaving():
   run_lift(mk_opts(), consumer_inst, consumer_ft, core_consumer, lambda:[], lambda _:())
 
 
+def test_sync_ignores_backpressure():
+  store = Store()
+  sync_opts = mk_opts(async_ = False)
+  async_opts = mk_opts(async_ = True)
+
+  callee_inst = ComponentInstance(store)
+
+  async_ft = FuncType([U32Type(), FutureType(None)],[U32Type()], async_ = True)
+  def core_callee1(thread, args):
+    [i,fut] = args
+    [ret] = canon_future_read(FutureType(None), sync_opts, thread, fut, 0xdeadbeef)
+    assert(ret == CopyResult.COMPLETED)
+    return [42 + i]
+  async_callee = partial(canon_lift, sync_opts, callee_inst, async_ft, core_callee1)
+
+  sync_ft = FuncType([U32Type()], [U32Type()])
+  def core_callee2(thread, args):
+    [i] = args
+    return [84 + i]
+  sync_callee = partial(canon_lift, sync_opts, callee_inst, sync_ft, core_callee2)
+
+  caller_inst = ComponentInstance(store)
+  caller_ft = FuncType([], [], async_ = True)
+  caller_mem = bytearray(24)
+  caller_opts = mk_opts(memory = caller_mem, async_ = True)
+  def core_caller(thread, args):
+    assert(len(args) == 0)
+
+    [packed] = canon_future_new(FutureType(None), thread)
+    rfut,wfut = unpack_new_ends(packed)
+
+    retp1 = 4
+    [ret] = canon_lower(caller_opts, async_ft, async_callee, thread, [1, rfut, retp1])
+    state,subi = unpack_result(ret)
+    assert(state == Subtask.State.STARTED)
+
+    retp2 = 8
+    [ret] = canon_lower(caller_opts, sync_ft, sync_callee, thread, [2, retp2])
+    assert(ret == Subtask.State.RETURNED)
+    assert(caller_mem[retp2] == 86)
+
+    [ret] = canon_future_write(FutureType(None), sync_opts, thread, wfut, 0xdeadbeef)
+    assert(ret == CopyResult.COMPLETED)
+
+    [seti] = canon_waitable_set_new(thread)
+    [] = canon_waitable_join(thread, subi, seti)
+    retp3 = 12
+    [event] = canon_waitable_set_wait(True, caller_mem, thread, seti, retp3)
+    assert(event == EventCode.SUBTASK)
+    assert(caller_mem[retp3+0] == subi)
+    assert(caller_mem[retp3+4] == Subtask.State.RETURNED)
+
+    [] = canon_subtask_drop(thread, subi)
+    return []
+
+  run_lift(mk_opts(), caller_inst, caller_ft, core_caller, lambda:[], lambda _:())
+
 def test_async_to_sync():
   store = Store()
   producer_opts = CanonicalOptions()
   producer_inst = ComponentInstance(store)
 
-  producer_ft = FuncType([],[])
+  producer_ft = FuncType([],[], async_ = True)
   fut = RacyBool(False)
   producer1_done = False
   def producer1_core(thread, args):
@@ -977,9 +1005,9 @@ def test_async_to_sync():
 
   consumer_heap = Heap(20)
   consumer_opts = mk_opts(consumer_heap.memory)
-  consumer_opts.sync = False
+  consumer_opts.async_ = True
 
-  consumer_ft = FuncType([],[U8Type()])
+  consumer_ft = FuncType([],[U8Type()], async_ = True)
   def consumer(thread, args):
     assert(len(args) == 0)
 
@@ -1003,6 +1031,8 @@ def test_async_to_sync():
 
     remain = [subi1, subi2]
     while remain:
+      [ret] = canon_thread_yield(True, thread)
+      assert(ret == 0)
       retp = 8
       [event] = canon_waitable_set_poll(True, consumer_heap.memory, thread, seti, retp)
       if event == EventCode.NONE:
@@ -1036,10 +1066,10 @@ def test_async_to_sync():
 def test_async_backpressure():
   store = Store()
   producer_opts = CanonicalOptions()
-  producer_opts.sync = False
+  producer_opts.async_ = True
   producer_inst = ComponentInstance(store)
 
-  producer_ft = FuncType([],[])
+  producer_ft = FuncType([],[], async_ = True)
   fut = RacyBool(False)
   producer1_done = False
   def producer1_core(thread, args):
@@ -1063,9 +1093,9 @@ def test_async_backpressure():
   producer2 = partial(canon_lift, producer_opts, producer_inst, producer_ft, producer2_core)
 
   consumer_heap = Heap(20)
-  consumer_opts = mk_opts(consumer_heap.memory, sync = False)
+  consumer_opts = mk_opts(consumer_heap.memory, async_ = True)
 
-  consumer_ft = FuncType([],[U8Type()])
+  consumer_ft = FuncType([],[U8Type()], async_ = True)
   def consumer(thread, args):
     assert(len(args) == 0)
 
@@ -1120,9 +1150,9 @@ def test_async_backpressure():
 def test_sync_using_wait():
   store = Store()
   hostcall_opts = mk_opts()
-  hostcall_opts.sync = False
+  hostcall_opts.async_ = True
   hostcall_inst = ComponentInstance(store)
-  ft = FuncType([], [])
+  ft = FuncType([], [], async_ = True)
 
   def core_hostcall_pre(fut, thread, args):
     thread.suspend_until(fut.is_set)
@@ -1137,7 +1167,7 @@ def test_sync_using_wait():
 
   lower_heap = Heap(20)
   lower_opts = mk_opts(lower_heap.memory)
-  lower_opts.sync = False
+  lower_opts.async_ = True
 
   def core_func(thread, args):
     [ret] = canon_lower(lower_opts, ft, hostcall1, thread, [])
@@ -1341,8 +1371,8 @@ def test_eager_stream_completion():
   ft = FuncType([StreamType(U8Type())], [StreamType(U8Type())])
   inst = ComponentInstance(store)
   mem = bytearray(20)
-  opts = mk_opts(memory=mem, sync=False)
-  sync_opts = mk_opts(memory=mem, sync=True)
+  opts = mk_opts(memory=mem, async_=True)
+  sync_opts = mk_opts(memory=mem, async_=False)
 
   def host_import(caller, on_start, on_resolve):
     args = on_start()
@@ -1424,8 +1454,8 @@ def test_async_stream_ops():
   ft = FuncType([StreamType(U8Type())], [StreamType(U8Type())])
   inst = ComponentInstance(store)
   mem = bytearray(24)
-  opts = mk_opts(memory=mem, sync=False)
-  sync_opts = mk_opts(memory=mem, sync=True)
+  opts = mk_opts(memory=mem, async_=True)
+  sync_opts = mk_opts(memory=mem, async_=False)
 
   host_import_incoming = None
   host_import_outgoing = None
@@ -1480,7 +1510,7 @@ def test_async_stream_ops():
     [seti] = canon_waitable_set_new(thread)
     [] = canon_waitable_join(thread, rsi1, seti)
     definitions.throw_it = True
-    [event] = canon_waitable_set_wait(True, mem, thread, seti, retp)
+    [event] = canon_waitable_set_wait(True, mem, thread, seti, retp) ##
     assert(event == EventCode.STREAM_READ)
     assert(mem[retp+0] == rsi1)
     result,n = unpack_result(mem[retp+4])
@@ -1574,7 +1604,7 @@ def test_receive_own_stream():
   store = Store()
   inst = ComponentInstance(store)
   mem = bytearray(20)
-  opts = mk_opts(memory=mem, sync=False)
+  opts = mk_opts(memory=mem, async_=True)
 
   host_ft = FuncType([StreamType(U8Type())], [StreamType(U8Type())])
   def host_import(caller, on_start, on_resolve):
@@ -1597,23 +1627,22 @@ def test_receive_own_stream():
     assert(ret == Subtask.State.RETURNED)
     rsi2 = int.from_bytes(mem[retp : retp+4], 'little', signed=False)
     assert(rsi2 == 2)
-    try:
-      canon_stream_cancel_write(StreamType(U8Type()), True, thread, wsi)
-    except Trap:
-      pass
+    [ret] = canon_stream_cancel_write(StreamType(U8Type()), False, thread, wsi)
+    result,n = unpack_result(ret)
+    assert(result == CopyResult.CANCELLED and n == 0)
     [] = canon_stream_drop_writable(StreamType(U8Type()), thread, wsi)
     return []
 
   def on_start(): return []
   def on_resolve(results): assert(len(results) == 0)
-  ft = FuncType([],[])
+  ft = FuncType([],[], async_ = True)
   run_lift(mk_opts(), inst, ft, core_func, on_start, on_resolve)
 
 
 def test_host_partial_reads_writes():
   store = Store()
   mem = bytearray(20)
-  opts = mk_opts(memory=mem, sync=False)
+  opts = mk_opts(memory=mem, async_=True)
 
   src = HostSource(U8Type(), [1,2,3,4], chunk=2, destroy_if_empty = False)
   source_ft = FuncType([], [StreamType(U8Type())])
@@ -1689,7 +1718,7 @@ def test_host_partial_reads_writes():
   inst = ComponentInstance(Store())
   def on_start(): return []
   def on_resolve(results): assert(len(results) == 0)
-  ft = FuncType([],[])
+  ft = FuncType([],[], async_ = True)
   run_lift(opts2, inst, ft, core_func, on_start, on_resolve)
 
 
@@ -1699,7 +1728,7 @@ def test_wasm_to_wasm_stream():
 
   inst1 = ComponentInstance(store)
   mem1 = bytearray(24)
-  opts1 = mk_opts(memory=mem1, sync=False)
+  opts1 = mk_opts(memory=mem1, async_=True)
   ft1 = FuncType([], [StreamType(U8Type())])
   def core_func1(thread, args):
     assert(not args)
@@ -1719,7 +1748,7 @@ def test_wasm_to_wasm_stream():
 
     [ret] = canon_stream_write(StreamType(U8Type()), opts1, thread, wsi, 0, 0)
     assert(ret == definitions.BLOCKED)
-    [ret] = canon_stream_cancel_write(StreamType(U8Type()), False, thread, wsi)
+    [ret] = canon_stream_cancel_write(StreamType(U8Type()), True, thread, wsi)
     result,n = unpack_result(ret)
     assert(n == 0 and result == CopyResult.CANCELLED)
 
@@ -1764,7 +1793,7 @@ def test_wasm_to_wasm_stream():
   inst2 = ComponentInstance(store)
   heap2 = Heap(24)
   mem2 = heap2.memory
-  opts2 = mk_opts(memory=heap2.memory, realloc=heap2.realloc, sync=False)
+  opts2 = mk_opts(memory=heap2.memory, realloc=heap2.realloc, async_=True)
   ft2 = FuncType([], [])
   def core_func2(thread, args):
     assert(not args)
@@ -1830,7 +1859,7 @@ def test_wasm_to_wasm_stream_empty():
 
   inst1 = ComponentInstance(store)
   mem1 = bytearray(24)
-  opts1 = mk_opts(memory=mem1, sync=False)
+  opts1 = mk_opts(memory=mem1, async_=True)
   ft1 = FuncType([], [StreamType(None)])
   def core_func1(thread, args):
     assert(not args)
@@ -1875,7 +1904,7 @@ def test_wasm_to_wasm_stream_empty():
   inst2 = ComponentInstance(store)
   heap2 = Heap(10)
   mem2 = heap2.memory
-  opts2 = mk_opts(memory=heap2.memory, realloc=heap2.realloc, sync=False)
+  opts2 = mk_opts(memory=heap2.memory, realloc=heap2.realloc, async_=True)
   ft2 = FuncType([], [])
   def core_func2(thread, args):
     assert(not args)
@@ -1925,7 +1954,7 @@ def test_cancel_copy():
   store = Store()
   inst = ComponentInstance(store)
   mem = bytearray(24)
-  lower_opts = mk_opts(memory=mem, sync=False)
+  lower_opts = mk_opts(memory=mem, async_=True)
 
   host_ft1 = FuncType([StreamType(U8Type())],[])
   host_sink = None
@@ -1959,7 +1988,7 @@ def test_cancel_copy():
     host_sink.set_remain(2)
     got = host_sink.consume(2)
     assert(got == [0xa, 0xb])
-    [ret] = canon_stream_cancel_write(StreamType(U8Type()), True, thread, wsi)
+    [ret] = canon_stream_cancel_write(StreamType(U8Type()), False, thread, wsi)
     result,n = unpack_result(ret)
     assert(n == 2 and result == CopyResult.COMPLETED)
     [] = canon_stream_drop_writable(StreamType(U8Type()), thread, wsi)
@@ -1976,7 +2005,7 @@ def test_cancel_copy():
     host_sink.set_remain(2)
     got = host_sink.consume(2)
     assert(got == [1, 2])
-    [ret] = canon_stream_cancel_write(StreamType(U8Type()), False, thread, wsi)
+    [ret] = canon_stream_cancel_write(StreamType(U8Type()), True, thread, wsi)
     result,n = unpack_result(ret)
     assert(n == 2 and result == CopyResult.COMPLETED)
     [] = canon_stream_drop_writable(StreamType(U8Type()), thread, wsi)
@@ -1989,7 +2018,7 @@ def test_cancel_copy():
     rsi = mem[retp]
     [ret] = canon_stream_read(StreamType(U8Type()), lower_opts, thread, rsi, 0, 4)
     assert(ret == definitions.BLOCKED)
-    [ret] = canon_stream_cancel_read(StreamType(U8Type()), True, thread, rsi)
+    [ret] = canon_stream_cancel_read(StreamType(U8Type()), False, thread, rsi)
     result,n = unpack_result(ret)
     assert(n == 0 and result == CopyResult.CANCELLED)
     [] = canon_stream_drop_readable(StreamType(U8Type()), thread, rsi)
@@ -2000,7 +2029,7 @@ def test_cancel_copy():
     [ret] = canon_stream_read(StreamType(U8Type()), lower_opts, thread, rsi, 0, 4)
     assert(ret == definitions.BLOCKED)
     host_source.block_cancel()
-    [ret] = canon_stream_cancel_read(StreamType(U8Type()), False, thread, rsi)
+    [ret] = canon_stream_cancel_read(StreamType(U8Type()), True, thread, rsi)
     assert(ret == definitions.BLOCKED)
     host_source.write([7,8])
     host_source.unblock_cancel()
@@ -2017,7 +2046,8 @@ def test_cancel_copy():
 
     return []
 
-  run_lift(lift_opts, inst, FuncType([],[]), core_func, lambda:[], lambda _:())
+  caller_ft = FuncType([], [], async_ = True)
+  run_lift(lift_opts, inst, caller_ft, core_func, lambda:[], lambda _:())
 
 
 class HostFutureSink:
@@ -2074,7 +2104,7 @@ def test_futures():
   store = Store()
   inst = ComponentInstance(store)
   mem = bytearray(24)
-  lower_opts = mk_opts(memory=mem, sync=False)
+  lower_opts = mk_opts(memory=mem, async_=True)
 
   host_ft1 = FuncType([FutureType(U8Type())],[FutureType(U8Type())])
   def host_func(caller, on_start, on_resolve):
@@ -2138,7 +2168,7 @@ def test_futures():
     while not thread.task.inst.table.get(rfi).has_pending_event():
       canon_thread_yield(True, thread)
 
-    [ret] = canon_future_cancel_read(FutureType(U8Type()), True, thread, rfi)
+    [ret] = canon_future_cancel_read(FutureType(U8Type()), False, thread, rfi)
     assert(ret == CopyResult.COMPLETED)
     assert(mem[readp] == 43)
 
@@ -2156,16 +2186,17 @@ def test_futures():
 
     return []
 
-  run_lift(lift_opts, inst, FuncType([],[]), core_func, lambda:[], lambda _:())
+  caller_ft = FuncType([], [], async_ = True)
+  run_lift(lift_opts, inst, caller_ft, core_func, lambda:[], lambda _:())
 
 
 def test_cancel_subtask():
   store = Store()
-  ft = FuncType([U8Type()], [U8Type()])
+  ft = FuncType([U8Type()], [U8Type()], async_ = True)
 
   callee_heap = Heap(10)
-  callee_opts = mk_opts(callee_heap.memory, sync = False)
-  sync_callee_opts = mk_opts(callee_heap.memory, sync = True)
+  callee_opts = mk_opts(callee_heap.memory, async_ = True)
+  sync_callee_opts = mk_opts(callee_heap.memory, async_ = False)
   callee_inst = ComponentInstance(store)
 
   def core_callee1(thread, args):
@@ -2247,7 +2278,7 @@ def test_cancel_subtask():
     [ret] = canon_lower(callee_opts, ft, host_import5, thread, [42, 0])
     state,subi = unpack_result(ret)
     assert(state == Subtask.State.STARTED)
-    [ret] = canon_subtask_cancel(True, thread, subi)
+    [ret] = canon_subtask_cancel(False, thread, subi)
     assert(ret == Subtask.State.RETURNED)
     [] = canon_task_return(thread, [U8Type()], callee_opts, [44])
     return []
@@ -2261,11 +2292,11 @@ def test_cancel_subtask():
       [ret] = canon_thread_switch_to(True, thread, mainthreadi)
       assert(ret == SuspendResult.CANCELLED)
       [ret] = canon_thread_switch_to(True, thread, mainthreadi)
-      assert(ret == SuspendResult.COMPLETED)
+      assert(ret == SuspendResult.NOT_CANCELLED)
       [] = canon_task_return(thread, [U8Type()], callee_opts, [45])
     else:
       [ret] = canon_thread_switch_to(False, thread, mainthreadi)
-      assert(ret == SuspendResult.COMPLETED)
+      assert(ret == SuspendResult.NOT_CANCELLED)
     return []
   cthread_func = partial(thread_func, True)
   ncthread_func = partial(thread_func, False)
@@ -2280,18 +2311,18 @@ def test_cancel_subtask():
 
     [threadi1] = canon_thread_new_indirect(core_ft, core_ftbl, thread, ncfi, mainthreadi)
     [ret] = canon_thread_switch_to(True, thread, threadi1)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     [threadi2] = canon_thread_new_indirect(core_ft, core_ftbl, thread, cfi, mainthreadi)
     [ret] = canon_thread_switch_to(True, thread, threadi2)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     [threadi3] = canon_thread_new_indirect(core_ft, core_ftbl, thread, ncfi, mainthreadi)
     [ret] = canon_thread_switch_to(True, thread, threadi3)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     [ret] = canon_thread_suspend(False, thread)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     [] = canon_thread_resume_later(thread, threadi1)
     [] = canon_thread_resume_later(thread, threadi2)
@@ -2300,7 +2331,7 @@ def test_cancel_subtask():
   callee6 = partial(canon_lift, callee_opts, callee_inst, ft, core_callee6)
 
   caller_heap = Heap(20)
-  caller_opts = mk_opts(caller_heap.memory, sync = False)
+  caller_opts = mk_opts(caller_heap.memory, async_ = True)
   caller_inst = ComponentInstance(store)
 
   def core_caller(thread, args):
@@ -2316,9 +2347,9 @@ def test_cancel_subtask():
     [ret] = canon_lower(caller_opts, ft, callee1, thread, [13, 0])
     state,subi2 = unpack_result(ret)
     assert(state == Subtask.State.STARTING)
-    [ret] = canon_subtask_cancel(True, thread, subi2)
+    [ret] = canon_subtask_cancel(False, thread, subi2)
     assert(ret == Subtask.State.CANCELLED_BEFORE_STARTED)
-    [ret] = canon_subtask_cancel(False, thread, subi1)
+    [ret] = canon_subtask_cancel(True, thread, subi1)
     assert(ret == Subtask.State.CANCELLED_BEFORE_STARTED)
     callee_inst.backpressure = False
 
@@ -2342,19 +2373,19 @@ def test_cancel_subtask():
     assert(state == Subtask.State.STARTED)
 
     caller_heap.memory[0] = 13
-    [ret] = canon_subtask_cancel(True, thread, subi1)
+    [ret] = canon_subtask_cancel(False, thread, subi1)
     assert(ret == Subtask.State.RETURNED)
     assert(caller_heap.memory[0] == 42)
     [] = canon_subtask_drop(thread, subi1)
 
     caller_heap.memory[0] = 13
-    [ret] = canon_subtask_cancel(False, thread, subi2)
+    [ret] = canon_subtask_cancel(True, thread, subi2)
     assert(ret == Subtask.State.CANCELLED_BEFORE_RETURNED)
     assert(caller_heap.memory[0] == 13)
     [] = canon_subtask_drop(thread, subi2)
 
     caller_heap.memory[0] = 13
-    [ret] = canon_subtask_cancel(False, thread, subi3)
+    [ret] = canon_subtask_cancel(True, thread, subi3)
     assert(ret == definitions.BLOCKED)
     assert(caller_heap.memory[0] == 13)
     [] = canon_waitable_join(thread, subi3, seti)
@@ -2367,13 +2398,13 @@ def test_cancel_subtask():
     [] = canon_subtask_drop(thread, subi3)
 
     caller_heap.memory[0] = 13
-    [ret] = canon_subtask_cancel(True, thread, subi3_2)
+    [ret] = canon_subtask_cancel(False, thread, subi3_2)
     assert(ret == Subtask.State.RETURNED)
     assert(caller_heap.memory[0] == 43)
     [] = canon_subtask_drop(thread, subi3_2)
 
     caller_heap.memory[0] = 13
-    [ret] = canon_subtask_cancel(False, thread, subi4)
+    [ret] = canon_subtask_cancel(True, thread, subi4)
     assert(ret == definitions.BLOCKED)
     assert(caller_heap.memory[0] == 13)
     [] = canon_waitable_join(thread, subi4, seti)
@@ -2385,7 +2416,7 @@ def test_cancel_subtask():
     [] = canon_subtask_drop(thread, subi4)
 
     caller_heap.memory[0] = 13
-    [ret] = canon_subtask_cancel(True, thread, subi4_2)
+    [ret] = canon_subtask_cancel(False, thread, subi4_2)
     assert(ret == Subtask.State.CANCELLED_BEFORE_RETURNED)
     assert(caller_heap.memory[0] == 13)
     [] = canon_subtask_drop(thread, subi4_2)
@@ -2396,7 +2427,7 @@ def test_cancel_subtask():
     assert(state == Subtask.State.STARTED)
     while caller_inst.table.get(subi).state == Subtask.State.STARTED:
       [_] = canon_thread_yield(True, thread)
-    [ret] = canon_subtask_cancel(False, thread, subi)
+    [ret] = canon_subtask_cancel(True, thread, subi)
     assert(ret == Subtask.State.RETURNED)
     assert(caller_heap.memory[0] == 83)
     [] = canon_subtask_drop(thread, subi)
@@ -2405,7 +2436,7 @@ def test_cancel_subtask():
     [ret] = canon_lower(caller_opts, ft, callee3, thread, [0, 0])
     state,subi = unpack_result(ret)
     assert(state == Subtask.State.STARTED)
-    [ret] = canon_subtask_cancel(False, thread, subi)
+    [ret] = canon_subtask_cancel(True, thread, subi)
     assert(ret == Subtask.State.CANCELLED_BEFORE_RETURNED)
     assert(caller_heap.memory[0] == 13)
     [] = canon_subtask_drop(thread, subi)
@@ -2414,7 +2445,7 @@ def test_cancel_subtask():
     [ret] = canon_lower(caller_opts, ft, callee4, thread, [0, 0])
     state,subi = unpack_result(ret)
     assert(state == Subtask.State.STARTED)
-    [ret] = canon_subtask_cancel(False, thread, subi)
+    [ret] = canon_subtask_cancel(True, thread, subi)
     assert(ret == definitions.BLOCKED)
     assert(caller_heap.memory[0] == 13)
     host_fut4.set()
@@ -2430,7 +2461,7 @@ def test_cancel_subtask():
     [ret] = canon_lower(caller_opts, ft, callee5, thread, [13, 0])
     state,subi = unpack_result(ret)
     assert(state == Subtask.State.STARTED)
-    [ret] = canon_subtask_cancel(False, thread, subi)
+    [ret] = canon_subtask_cancel(True, thread, subi)
     assert(ret == definitions.BLOCKED)
     assert(caller_heap.memory[0] == 13)
     host_fut5.set()
@@ -2447,7 +2478,7 @@ def test_cancel_subtask():
     state,subi = unpack_result(ret)
     assert(state == Subtask.State.STARTED)
 
-    [ret] = canon_subtask_cancel(False, thread, subi)
+    [ret] = canon_subtask_cancel(True, thread, subi)
     assert(ret == definitions.BLOCKED)
 
     [] = canon_waitable_join(thread, subi, seti)
@@ -2476,46 +2507,46 @@ def test_cancel_subtask():
   assert(got[0] == 42)
 
 
-def test_self_empty():
+def test_self_copy(elemt):
   store = Store()
   inst = ComponentInstance(store)
-  mem = bytearray(24)
-  sync_opts = mk_opts(memory=mem, sync=True)
-  async_opts = mk_opts(memory=mem, sync=False)
+  mem = bytearray(40)
+  sync_opts = mk_opts(memory=mem, async_=False)
+  async_opts = mk_opts(memory=mem, async_=True)
 
-  ft = FuncType([],[])
+  ft = FuncType([], [], async_ = True)
   def core_func(thread, args):
     [seti] = canon_waitable_set_new(thread)
 
-    [packed] = canon_future_new(FutureType(None), thread)
+    [packed] = canon_future_new(FutureType(elemt), thread)
     rfi,wfi = unpack_new_ends(packed)
 
-    [ret] = canon_future_write(FutureType(None), async_opts, thread, wfi, 0xdeadbeef)
+    [ret] = canon_future_write(FutureType(elemt), async_opts, thread, wfi, 0)
     assert(ret == definitions.BLOCKED)
 
-    [ret] = canon_future_read(FutureType(None), async_opts, thread, rfi, 0xdeadbeef)
+    [ret] = canon_future_read(FutureType(elemt), async_opts, thread, rfi, 0)
     assert(ret == CopyResult.COMPLETED)
-    [] = canon_future_drop_readable(FutureType(None), thread, rfi)
+    [] = canon_future_drop_readable(FutureType(elemt), thread, rfi)
 
     [] = canon_waitable_join(thread, wfi, seti)
     [event] = canon_waitable_set_wait(True, mem, thread, seti, 0)
     assert(event == EventCode.FUTURE_WRITE)
     assert(mem[0] == wfi)
     assert(mem[4] == CopyResult.COMPLETED)
-    [] = canon_future_drop_writable(FutureType(None), thread, wfi)
+    [] = canon_future_drop_writable(FutureType(elemt), thread, wfi)
 
-    [packed] = canon_stream_new(StreamType(None), thread)
+    [packed] = canon_stream_new(StreamType(elemt), thread)
     rsi,wsi = unpack_new_ends(packed)
-    [ret] = canon_stream_write(StreamType(None), async_opts, thread, wsi, 10000, 3)
+    [ret] = canon_stream_write(StreamType(elemt), async_opts, thread, wsi, 0, 3)
     assert(ret == definitions.BLOCKED)
 
-    [ret] = canon_stream_read(StreamType(None), async_opts, thread, rsi, 2000, 1)
+    [ret] = canon_stream_read(StreamType(elemt), async_opts, thread, rsi, 0, 1)
     result,n = unpack_result(ret)
     assert(n == 1 and result == CopyResult.COMPLETED)
-    [ret] = canon_stream_read(StreamType(None), async_opts, thread, rsi, 2000, 4)
+    [ret] = canon_stream_read(StreamType(elemt), async_opts, thread, rsi, 0, 4)
     result,n = unpack_result(ret)
     assert(n == 2 and result == CopyResult.COMPLETED)
-    [] = canon_stream_drop_readable(StreamType(None), thread, rsi)
+    [] = canon_stream_drop_readable(StreamType(elemt), thread, rsi)
 
     [] = canon_waitable_join(thread, wsi, seti)
     [event] = canon_waitable_set_wait(True, mem, thread, seti, 0)
@@ -2524,7 +2555,7 @@ def test_self_empty():
     result,n = unpack_result(mem[4])
     assert(result == CopyResult.DROPPED)
     assert(n == 3)
-    [] = canon_stream_drop_writable(StreamType(None), thread, wsi)
+    [] = canon_stream_drop_writable(StreamType(elemt), thread, wsi)
 
     [] = canon_waitable_set_drop(thread, seti)
     return []
@@ -2535,7 +2566,7 @@ def test_self_empty():
 def test_async_flat_params():
   store = Store()
   heap = Heap(1000)
-  opts = mk_opts(heap.memory, 'utf8', heap.realloc, sync = False)
+  opts = mk_opts(heap.memory, 'utf8', heap.realloc, async_ = True)
 
   ft1 = FuncType([F32Type(), F64Type(), U32Type(), S64Type()],[])
   def f1(caller, on_start, on_resolve):
@@ -2598,7 +2629,7 @@ def test_threads():
   def thread_func2(thread, args):
     [mainthreadi] = args
     [ret] = canon_thread_yield_to(True, thread, mainthreadi)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
     return []
   fi2 = ftbl.add(CoreFuncRef(ft, thread_func2))
 
@@ -2611,10 +2642,10 @@ def test_threads():
   def thread_func4(thread, args):
     [ptr] = args
     [ret] = canon_thread_yield(False, thread)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
     mem[ptr] = mem[ptr] + 1
     [ret] = canon_thread_yield(False, thread)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
     mem[ptr] = mem[ptr] + 1
     return []
   fi4 = ftbl.add(CoreFuncRef(ft, thread_func4))
@@ -2626,16 +2657,16 @@ def test_threads():
 
     [threadi] = canon_thread_new_indirect(ft, ftbl, thread, fi1, 13)
     [ret] = canon_thread_yield_to(True, thread, threadi)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     [threadi] = canon_thread_new_indirect(ft, ftbl, thread, fi2, mainthreadi)
     [ret] = canon_thread_switch_to(True, thread, threadi)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     [threadi] = canon_thread_new_indirect(ft, ftbl, thread, fi3, mainthreadi)
     [] = canon_thread_resume_later(thread, threadi)
     [ret] = canon_thread_suspend(True, thread)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
 
     ptr = 4
     mem[ptr] = 0
@@ -2652,15 +2683,16 @@ def test_threads():
     nonlocal result
     [result] = v
 
-  run_lift(opts, inst, FuncType([], [U8Type()]), core_func, lambda:[], on_resolve)
+  caller_ft = FuncType([], [U8Type()], async_ = True)
+  run_lift(opts, inst, caller_ft, core_func, lambda:[], on_resolve)
   assert(result == 42)
 
 def test_thread_cancel_callback():
   store = Store()
   producer_inst = ComponentInstance(store)
-  producer_ft = FuncType([],[U32Type()])
+  producer_ft = FuncType([], [U32Type()], async_ = True)
 
-  producer_opts1 = mk_opts(sync = False)
+  producer_opts1 = mk_opts(async_ = True)
   def core_producer1(thread, args):
     assert(not args)
     return [CallbackCode.YIELD]
@@ -2672,11 +2704,11 @@ def test_thread_cancel_callback():
   producer_opts1.callback = core_producer_callback1
   producer_callee1 = partial(canon_lift, producer_opts1, producer_inst, producer_ft, core_producer1)
 
-  producer_opts2 = mk_opts(sync = False)
+  producer_opts2 = mk_opts(async_ = True)
   def core_producer2(thread, args):
     assert(not args)
     [ret] = canon_thread_yield(False, thread)
-    assert(ret == SuspendResult.COMPLETED)
+    assert(ret == SuspendResult.NOT_CANCELLED)
     [] = canon_task_return(thread, [U32Type()], producer_opts2, [43])
     return [CallbackCode.EXIT]
   def core_producer_callback2(thread, args):
@@ -2685,9 +2717,9 @@ def test_thread_cancel_callback():
   producer_callee2 = partial(canon_lift, producer_opts2, producer_inst, producer_ft, core_producer2)
 
   consumer_inst = ComponentInstance(store)
-  consumer_ft = FuncType([], [])
+  consumer_ft = FuncType([], [], async_ = True)
   consumer_mem = bytearray(24)
-  consumer_opts = mk_opts(consumer_mem, sync = False)
+  consumer_opts = mk_opts(consumer_mem, async_ = True)
 
   def core_consumer(thread, args):
     assert(len(args) == 0)
@@ -2702,7 +2734,7 @@ def test_thread_cancel_callback():
     state,subi2 = unpack_result(ret)
     assert(state == Subtask.State.STARTED)
 
-    [ret] = canon_subtask_cancel(False, thread, subi1)
+    [ret] = canon_subtask_cancel(True, thread, subi1)
     assert(ret == definitions.BLOCKED)
 
     retp3 = 16
@@ -2730,6 +2762,7 @@ test_handles()
 test_async_to_async()
 test_async_callback()
 test_callback_interleaving()
+test_sync_ignores_backpressure()
 test_async_to_sync()
 test_async_backpressure()
 test_sync_using_wait()
@@ -2743,7 +2776,9 @@ test_wasm_to_wasm_stream_empty()
 test_cancel_copy()
 test_futures()
 test_cancel_subtask()
-test_self_empty()
+test_self_copy(None)
+test_self_copy(U8Type())
+test_self_copy(F64Type())
 test_async_flat_params()
 test_threads()
 test_thread_cancel_callback()
